#  ============================================================================
#
#  Copyright (C) 2007-2011 Conceptive Engineering bvba. All rights reserved.
#  www.conceptive.be / project-camelot@conceptive.be
#
#  This file is part of the Camelot Library.
#
#  This file may be used under the terms of the GNU General Public
#  License version 2.0 as published by the Free Software Foundation
#  and appearing in the file license.txt included in the packaging of
#  this file.  Please review this information to ensure GNU
#  General Public Licensing requirements will be met.
#
#  If you are unsure which license is appropriate for your use, please
#  visit www.python-camelot.com or contact project-camelot@conceptive.be
#
#  This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
#  WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#
#  For use of this library in commercial applications, please contact
#  project-camelot@conceptive.be
#
#  ============================================================================

import logging

from PyQt4 import QtGui

LOGGER = logging.getLogger( 'camelot.admin.action' )

class ModelContext( object ):
    """
The Model context in which an action is running.  The model context can contain
reference to database sessions or other model related data. This object can not 
contain references to widgets as those belong strictly to the :class:`GuiContext`.    

.. attribute:: mode_name

    the name of the mode in which the action was triggered
    """
          
    def __init__( self ):
        self.mode_name = None
        
class GuiContext( object ):
    """
The GUI context in which an action is running.  This object can contain
references to widgets and other usefull information.  This object cannot
contain reference to anything database or model related, as those belong
strictly to the :class:`ModelContext`

.. attribute:: progress_dialog

    an instance of :class:`QtGui.QProgressDialog` or :keyword:`None`
    
.. attribute:: mode_name

    the name of the mode in which the action was triggered
    
.. attribute:: model_context

    a subclass of :class:`ModelContext` to be used in :meth:`create_model_context`
    as the type of object to return.
    """
    
    model_context = ModelContext
    
    def __init__( self ):
        self.progress_dialog = None
        self.mode_name = None
        
    def create_model_context( self ):
        """Create a :class:`ModelContext` filled with base information, 
        extracted from this GuiContext.  This function will be called in the
        GUI thread, so it should not access the model directly, but rather
        extract all information needed from te GUI to be available in the
        model.
        
        :return: a :class:`ModelContext`
        """
        context = self.model_context()
        context.mode_name = self.mode_name
        return context
        
    def copy( self ):
        """Create a copy of the GuiContext, this function is used
        to create new GuiContext's that are more specialized without
        modifying the original one."""
        new_context = self.__class__()
        new_context.progress_dialog = self.progress_dialog
        new_context.mode_name = self.mode_name
        return new_context
                    
class State( object ):
    """A state represents the appearance and behavior of the widget that
triggers the action.  When the objects in the model change, the 
:meth:`Action.get_state` method will be called, which should return the
updated state for the widget.

.. attribute:: verbose_name

    The name of the action as it will appear in the button, this defaults to
    the verbose_name of the action.
    
.. attribute:: icon

    The icon that represents the action, of type 
    :class:`camelot.view.art.Icon`, this defaults to the icon of the action.

.. attribute:: tooltip

    The tooltip as displayed to the user, this should be of type 
    :class:`camelot.core.utils.ugettext_lazy`, this defaults to the tooltip
    op the action.

.. attribute:: enabled

    :keyword:`True` if the widget should be enabled (the default), 
    :keyword:`False` otherwise
    
.. attribute:: visible

    :keyword:`True` if the widget should be visible (the default), 
    :keyword:`False` otherwise

.. attribute:: notification

    :keyword:`True` if the buttons should attract the attention of the user, 
    defaults to :keyword:`False`.

.. attribute:: modes

    The modes in which an action can be triggered, a list of :class:`Mode`
    objects.

    """
    
    def __init__( self ):
        self.verbose_name = None
        self.icon = None
        self.tooltip = None
        self.enabled = True
        self.visible = True
        self.notification = False
        self.modes = []

class Mode( object ):
    """A mode is a way in which an action can be triggered, a print action could
be triggered as 'Export to PDF' or 'Export to Word'.  None always represents
the default mode.
    
.. attribute:: name

    a string representing the mode to the developer and the authentication
    system.  this name will be used in the :class:`GuiContext`
    
.. attribute:: verbose_name

    The name shown to the user
    
.. attribute:: icon

    The icon of the mode
    """
    
    def __init__( self, name, verbose_name=None, icon=None):
        """
        :param name: the name of the mode, as it will be passed to the
            gui_run and model_run method
        :param verbose_name: the name shown to the user
        :param icon: the icon of the mode
        """
        self.name = name
        self.verbose_name = verbose_name or name.capitalize()
        self.icon = icon
        
    def render( self, parent ):
        """Create a :class:`QtGui.QAction` that can be used to enable widget
        to trigger the action in a specific mode.  The data attribute of the
        action will contain the name of the mode.
        
        :return: a :class:`QtGui.QAction` class to use this mode
        """
        action = QtGui.QAction( parent )
        action.setData( self.name )
        action.setText( unicode(self.verbose_name) )
        action.setIconVisibleInMenu( False )
        return action
        
class ActionStep( object ):
    """A reusable part of an action.  Action step object can be yielded inside
the :meth:`model_run`.  When this happens, their :meth:`gui_run` method will
be called inside the *GUI thread*.  The :meth:`gui_run` can pop up a dialog
box or perform other GUI related tasks.

When the ActionStep is blocking, it will return control after the 
:meth:`gui_run` is finished, and the return value of :meth:`gui_run` will
be the result of the :keyword:`yield` statement.

When the ActionStep is not blocking, the :keyword:`yield` statement will
return immediately and the :meth:`model_run` will not be blocked.

.. attribute:: blocking

    a :keyword:`boolean` indicating if the ActionStep is blocking, defaults
    to :keyword:`True`
    """

    blocking = True
            
    def gui_run( self, gui_context ):
        """This method is called in the *GUI thread* upon execution of the
        action step.  The return value of this method is the result of the
        :keyword:`yield` statement in the *model thread*.
        
        The default behavior of this method is to call the model_run generator
        in the *model thread* until it is finished.
        
        :param gui_context:  An object of type 
            :class:`camelot.admin.action.GuiContext`, which is the context 
            of this action available in the *GUI thread*.  What is in the 
            context depends on how the action was called.
            
        this method will raise a :class:`camelot.core.exception.CancelRequest`
        exception, if the user canceled the operation.
        """
        from camelot.view.action_runner import ActionRunner
        runner = ActionRunner( self.model_run, gui_context )
        runner.exec_()
        
    def model_run( self, model_context = None ):
        """A generator that yields :class:`camelot.admin.action.ActionStep`
        objects.  This generator can be called in the *model thread*.
        
        :param context:  An object of type
            :class:`camelot.admin.action.ModelContext`, which is context 
            of this action available in the model_thread.  What is in the 
            context depends on how the action was called.
        """
        yield

class Action( ActionStep ):
    """An action has a set of attributes that define its appearance in the
GUI.  
    
.. attribute:: name

    The internal name of the action, this can be used to store preferences
    concerning the action in the settings

These attributes are used at the default values for the creation of a
:class:`camelot.admin.action.base.State` object that defines the appearance
of the action button.  Subclasses of :class:`Action` that require dynamic
values for these attributes can reimplement the :class:`Action.get_state`
method.

.. attribute:: verbose_name

    The name as displayed to the user, this should be of type 
    :class:`camelot.core.utils.ugettext_lazy`
    
.. attribute:: icon

    The icon that represents the action, of type 
    :class:`camelot.view.art.Icon`

.. attribute:: tooltip

    The tooltip as displayed to the user, this should be of type 
    :class:`camelot.core.utils.ugettext_lazy`

.. attribute:: modes

    The modes in which an action can be triggered, a list of :class:`Mode`
    objects.
    
For each of these attributes there is a corresponding getter method
which is used by the view.  Subclasses of :class:`Action` that require dynamic
values for these attributes can reimplement the getter methods.

.. attribute:: shortcut

    The shortcut that can be used to trigger the action, this should be of 
    type :class:`camelot.core.utils.ugettext_lazy`
    
An action has two important methods that can be reimplemented.  These are 
:meth:`model_run` for manipulations of the model and :meth:`gui_run` for
direct manipulations of the user interface without a need to access the model.
        """
    
    name = u'action'
    verbose_name = None
    icon = None
    tooltip = None
    shortcut = None 
    modes = []
    
    def get_name( self ):
        """
        :return: a string, by default the :attr:`name` attribute
        """
        return self.name
    
    def get_shortcut( self ):
        """
        :return: a :class:`camelot.core.utils.ugettext_lazy`, by default the 
        :attr:`shortcut` attribute
        """
        return self.shortcut
        
    def render( self, gui_context, parent ):
        """Create a widget to trigger the action.  Depending on the type of
        gui_context and parent, a different widget type might be returned.
        
        :param gui_context: the context available in the *GUI thread*, a
            subclass of :class:`camelot.action.GuiContext`
        :param parent: the parent :class:`QtGui.QWidget`
        :return: a :class:`QtGui.QWidget` which when triggered
            will execute the :meth:`gui_run` method.
        """
        from camelot.view.controls.action_widget import ActionLabel, ActionPushButton
        from camelot.view.workspace import DesktopBackground
        if isinstance( parent, DesktopBackground ):
            return ActionLabel( self, gui_context, parent )
        return ActionPushButton( self, gui_context, parent )
        
    def gui_run( self, gui_context ):
        """This method is called inside the GUI thread, by default it
        executes the :meth:`model_run` in the Model thread.
        
        :param gui_context: the context available in the *GUI thread*,
            of type :class:`GuiContext`
            
        """
        from camelot.view.controls.progress_dialog import ProgressDialog
        progress_dialog = None
        # only create a progress dialog if there is none yet
        if gui_context.progress_dialog == None:
            progress_dialog = ProgressDialog( unicode( self.verbose_name ) )
            gui_context.progress_dialog = progress_dialog
            progress_dialog.show()
        super(Action, self).gui_run( gui_context )
        # only close the progress dialog if it was created here
        if progress_dialog != None:
            progress_dialog.close()
            gui_context.progress_dialog = None
        
    def get_state( self, model_context ):
        """
        This method is called inside the Model thread to verify if
        the state of the action widget visible to the current user.
        
        :param model_context: the context available in the *Model thread*
        :return: an instance of :class:`camelot.action.base.State`
        """
        state = State()
        state.verbose_name = self.verbose_name
        state.icon = self.icon
        state.tooltip = self.tooltip
        state.modes = self.modes
        return state



#  ============================================================================
#
#  Copyright (C) 2007-2011 Conceptive Engineering bvba. All rights reserved.
#  www.conceptive.be / project-camelot@conceptive.be
#
#  This file is part of the Camelot Library.
#
#  This file may be used under the terms of the GNU General Public
#  License version 2.0 as published by the Free Software Foundation
#  and appearing in the file license.txt included in the packaging of
#  this file.  Please review this information to ensure GNU
#  General Public Licensing requirements will be met.
#
#  If you are unsure which license is appropriate for your use, please
#  visit www.python-camelot.com or contact project-camelot@conceptive.be
#
#  This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
#  WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#
#  For use of this library in commercial applications, please contact
#  project-camelot@conceptive.be
#
#  ============================================================================

from PyQt4.QtCore import Qt
from PyQt4.QtGui import QDialog, QFrame, QGridLayout, QLabel, QVBoxLayout, \
    QWidget, QLayout

from camelot.view.model_thread import gui_function

class HSeparator(QFrame):

    def __init__(self, parent=None):
        super(HSeparator, self).__init__(parent)
        self.setFrameStyle(QFrame.HLine | QFrame.Sunken)


class StandaloneWizardPage(QDialog):
    """A Standalone Wizard Page Dialog for quick configuration windows"""

    def __init__(self, window_title=None, parent=None, flags=Qt.WindowFlags(0)):
        super(StandaloneWizardPage, self).__init__(parent, flags)
        self.setWindowTitle(window_title or '')
        self.set_layouts()

    @gui_function
    def set_layouts(self):
        self._vlayout = QVBoxLayout()
        self._vlayout.setSpacing(0)
        self._vlayout.setContentsMargins(0,0,0,0)

        # needed in case we have a widget that changes the size
        # of the widget and can be hidden
        self._vlayout.setSizeConstraint(QLayout.SetFixedSize)

        banner_layout = QGridLayout()
        banner_layout.setColumnStretch(0, 1)
        banner_layout.addWidget(QLabel(), 0, 1, Qt.AlignRight)
        banner_layout.addLayout(QVBoxLayout(), 0, 0)

        # TODO: allow banner widget to be supplied
        banner_widget = QWidget()
        banner_widget.setLayout(banner_layout)

        self._vlayout.addWidget(banner_widget)
        self._vlayout.addWidget(HSeparator())
        self._vlayout.addWidget(QFrame(), 1)
        self._vlayout.addWidget(HSeparator())
        self._vlayout.addWidget(QWidget())
        self.setLayout(self._vlayout)

    def banner_widget(self):
        return self._vlayout.itemAt(0).widget()

    def main_widget(self):
        return self._vlayout.itemAt(2).widget()

    def buttons_widget(self):
        return self._vlayout.itemAt(4).widget()

    def banner_layout(self):
        return self.banner_widget().layout()

    def banner_logo_holder(self):
        return self.banner_layout().itemAtPosition(0, 1).widget()

    def banner_text_layout(self):
        return self.banner_layout().itemAtPosition(0, 0).layout()

    def set_banner_logo_pixmap(self, pixmap):
        self.banner_logo_holder().setPixmap(pixmap)

    def set_banner_title(self, title):
        title_widget = QLabel('<dt><b>%s</b></dt>' % title)
        self.banner_text_layout().insertWidget(0, title_widget)

    def set_banner_subtitle(self, subtitle):
        subtitle_widget = QLabel('<dd>%s</dd>' % subtitle)
        self.banner_text_layout().insertWidget(1, subtitle_widget)



#  ============================================================================
#
#  Copyright (C) 2007-2011 Conceptive Engineering bvba. All rights reserved.
#  www.conceptive.be / project-camelot@conceptive.be
#
#  This file is part of the Camelot Library.
#
#  This file may be used under the terms of the GNU General Public
#  License version 2.0 as published by the Free Software Foundation
#  and appearing in the file license.txt included in the packaging of
#  this file.  Please review this information to ensure GNU
#  General Public Licensing requirements will be met.
#
#  If you are unsure which license is appropriate for your use, please
#  visit www.python-camelot.com or contact project-camelot@conceptive.be
#
#  This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
#  WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#
#  For use of this library in commercial applications, please contact
#  project-camelot@conceptive.be
#
#  ============================================================================
from PyQt4 import QtGui
from PyQt4 import QtCore

from customeditor import AbstractCustomEditor

class LanguageEditor(QtGui.QComboBox, AbstractCustomEditor):
    """A ComboBox that shows a list of languages, the editor takes
    as its value the ISO code of the language"""

    editingFinished = QtCore.pyqtSignal()
    language_choices = []
    
    def __init__(self, parent=None, languages=[], field_name='language', **kwargs):
        """
        :param languages: a list of ISO codes with languages
        that are allowed in the combo box, if the list is empty, all languages
        are allowed (the default)
        """
        QtGui.QComboBox.__init__(self, parent)
        AbstractCustomEditor.__init__(self)
        self.setObjectName( field_name )
        self.index_by_language = dict()
        languages = [QtCore.QLocale(lang).language() for lang in languages]
        if not self.language_choices:
            for language in range(QtCore.QLocale.C, QtCore.QLocale.Chewa + 1):
                if languages and (language not in languages):
                    continue
                language_name = unicode( QtCore.QLocale.languageToString( language ))
                self.language_choices.append( (language, language_name ) )
            self.language_choices.sort(key=lambda x:x[1])
        for i, (language, language_name) in enumerate( self.language_choices ):
            self.addItem( language_name, QtCore.QVariant(language) )
            self.index_by_language[ language ] = i
        self.activated.connect( self._activated )

    @QtCore.pyqtSlot(int)
    def _activated(self, _index):
        self.editingFinished.emit()
            
    def set_value(self, value):
        value = AbstractCustomEditor.set_value(self, value)
        if value:
            locale = QtCore.QLocale( value )
            self.setCurrentIndex( self.index_by_language[locale.language()] )
            
    def get_value(self):
        from camelot.core.utils import variant_to_pyobject
        current_index = self.currentIndex()
        if current_index >= 0:
            language = variant_to_pyobject(self.itemData(self.currentIndex()))
            locale = QtCore.QLocale( language )
            value = unicode( locale.name() )
        else:
            value = None
        return AbstractCustomEditor.get_value(self) or value




# -*- coding: utf-8 -*-
from PyQt4 import QtGui

import logging
import unittest
import os
import time

from camelot.test import ModelThreadTestCase, EntityViewsTest, SchemaTest
from camelot.view.art import ColorScheme

from PyQt4 import QtGui, QtCore
from PyQt4.QtGui import *
from PyQt4.QtCore import *

logger = logging.getLogger('view.unittests')

static_images_path = os.path.join(os.path.dirname(__file__), '..', 'doc', 'sphinx', 'source', '_static')

def create_getter(getable):

    def getter():
        return getable

    return getter

#class ProxyEntityTest(ModelThreadTestCase):
#  """Test the functionality of the proxies to perform CRUD operations on stand
#  alone data"""
#  def setUp(self):
#    ModelThreadTestCase.setUp(self)
#    from camelot.model.authentication import Person, PartyAddressRoleType
#    from camelot.view.proxy.queryproxy import QueryTableProxy
#    from camelot.view.application_admin import ApplicationAdmin
#    self.app_admin = ApplicationAdmin()
#    self.block = self.mt.post_and_block
#    person_admin = self.app_admin.getEntityAdmin(Person)
#    party_address_role_type_admin = self.app_admin.getEntityAdmin(PartyAddressRoleType)
#    self.person_proxy = QueryTableProxy(person_admin, lambda:Person.query, person_admin.get_fields)
#    self.party_address_role_type_proxy = QueryTableProxy(party_address_role_type_admin, lambda:PartyAddressRoleType.query, party_address_role_type_admin.get_fields)
#    # get the columns of the proxy
#    self.person_columns = dict((c[0],i) for i,c in enumerate(self.block(lambda:self.person_proxy.getColumns())))
#    self.rows_before_insert = 0
#    self.rows_after_insert = 0
#    self.rows_after_delete = 0
#    self.new_person = None
#    # make sure nothing is running in the model thread any more
#    self.block(lambda:None)
#  def listPersons(self):
#    from camelot.model.authentication import Person
#
#    def create_list():
#      print '<persons>'
#      for p in Person.query.all():
#        print p.id, unicode(p)
#      print '</persons>'
#
#    self.block(create_list)
#  def numberOfPersons(self):
#    return self.block(lambda:self.person_proxy.getRowCount())
#  def insertNewPerson(self, valid):
#    from camelot.model.authentication import Person
#    self.rows_before_insert = self.numberOfPersons()
#    self.new_person = self.block(lambda:Person(first_name={True:u'test', False:None}[valid], last_name='test'))
#    self.person_proxy.insertRow(0, create_getter(self.new_person))
#    self.rows_after_insert = self.numberOfPersons()
#    self.assertTrue( self.rows_after_insert > self.rows_before_insert)
#  def updateNewPerson(self):
#    rows_before_update = self.numberOfPersons()
#    self.assertNotEqual( self.block(lambda:self.new_person.last_name), u'Testers')
#    index = self.person_proxy.index(self.rows_before_insert, self.person_columns['last_name'])
#    self.person_proxy.setData(index, lambda:u'Testers')
#    self.assertEqual( self.block(lambda:self.new_person.last_name), u'Testers')
#    self.assertEqual(rows_before_update, self.numberOfPersons())
#  def updateNewPersonToValid(self):
#    index = self.person_proxy.index(self.rows_before_insert, self.person_columns['first_name'])
#    self.person_proxy.setData(index, lambda:u'test')
#    index = self.person_proxy.index(self.rows_before_insert, self.person_columns['last_name'])
#    self.person_proxy.setData(index, lambda:u'test')
#  def updateNewPersonToInvalid(self):
#    index = self.person_proxy.index(self.rows_before_insert, self.person_columns['first_name'])
#    self.person_proxy.setData(index, lambda:None)
#    index = self.person_proxy.index(self.rows_before_insert, self.person_columns['last_name'])
#    self.person_proxy.setData(index, lambda:None)
#  def deleteNewPerson(self):
#    self.person_proxy.removeRow(self.rows_before_insert)
#    self.rows_after_delete = self.numberOfPersons()
#    self.assertTrue( self.rows_after_delete < self.rows_after_insert )
#  def testCreateDataWithoutRequiredFields(self):
#    """Create a record that has no required fields, so the creation
#    should be instantaneous as opposed to delayed until all required fields are filled"""
#    from camelot.model.authentication import PartyAddressRoleType
#    rows_before_insert = self.block(lambda:self.party_address_role_type_proxy.getRowCount())
#    new_party_address_role_type_proxy = self.block(lambda:PartyAddressRoleType())
#    self.party_address_role_type_proxy.insertRow(0, create_getter(new_party_address_role_type_proxy))
#    self.assertNotEqual( self.block(lambda:new_party_address_role_type_proxy.id), None )
#    rows_after_insert = self.block(lambda:self.party_address_role_type_proxy.getRowCount())
#    self.assertTrue( rows_after_insert > rows_before_insert)
#  def testCreateUpdateDeleteValidData(self):
#    self.insertNewPerson(valid=True)
#    self.assertTrue( self.block(lambda:self.new_person.id) != None )
#    self.updateNewPerson()
#    self.deleteNewPerson()
#  def testCreateUpdateDeleteInvalidData(self):
#    self.insertNewPerson(valid=False)
#    self.assertTrue( self.block(lambda:self.new_person.id) == None )
#    self.updateNewPerson()
#    self.deleteNewPerson()
#  def testCreateInvalidDataUpdateToValidAndDelete(self):
#    self.insertNewPerson(valid=False)
#    self.assertTrue( self.block(lambda:self.new_person.id) == None )
#    self.updateNewPersonToValid()
#    self.assertTrue( self.block(lambda:self.new_person.id) != None )
#    self.deleteNewPerson()
#  def testCreateValidDataUpdateToInvalidAndDelete(self):
#    self.insertNewPerson(valid=False)
#    self.updateNewPersonToInvalid()
#    self.deleteNewPerson()


#class ProxyOneToManyTest(ProxyEntityTest):
#  """Test the functionality of the proxies to perform CRUD operations on related
#  data"""
#  def setUp(self):
#    super(ProxyOneToManyTest, self).setUp()
#    from elixir import session
#    from camelot.view.proxy.queryproxy import QueryTableProxy
#    from camelot.model.authentication import Country, City, Address, PartyAddress
#    party_address_admin = self.app_admin.getEntityAdmin(PartyAddress)
#    self.party_address_proxy = QueryTableProxy(party_address_admin, PartyAddress.query, party_address_admin.get_fields)
#    self.party_address_columns = dict((c[0],i) for i,c in enumerate(self.block(lambda:self.party_address_proxy.getColumns())))
#    self.country = self.block(lambda:Country(code=u'BE', name=u'Belgium'))
#    self.city = self.block(lambda:City(code=u'2000', name=u'Antwerp', country=self.country))
#    self.address = self.block(lambda:Address(street1=u'Teststreet', city=self.city))
#    self.block(lambda:session.flush([self.country, self.city, self.address]))
#  def insertRelatedPartyAddress(self, valid):
#    from camelot.model.authentication import PartyAddress
#    index = self.person_proxy.index(self.rows_before_insert, self.person_columns['addresses'])
#    self.block(lambda:None)
#    self.related_party_address_proxy = self.person_proxy.data(index, Qt.EditRole).toPyObject()
#    self.block(lambda:None)
#    self.related_party_address_proxy = self.person_proxy.data(index, Qt.EditRole).toPyObject()
#    self.assertNotEqual(self.related_party_address_proxy, None)
#    self.party_address = self.block({True:lambda:PartyAddress(address=self.address), False:lambda:PartyAddress()}[valid])
#    self.related_party_address_proxy.insertRow(0, create_getter(self.party_address))
#    self.block(lambda:None)
#  def testCreateBothValidDataUpdateDelete(self):
#    self.insertNewPerson(valid=True)
#    self.insertRelatedPartyAddress(valid=True)
#    self.assertNotEqual(self.party_address.id, None)
#    self.deleteNewPerson()
#  def testCreateValidPersonInvalidPartyAddressUpdateDelete(self):
#    self.insertNewPerson(valid=True)
#    self.insertRelatedPartyAddress(valid=False)
#    self.assertEqual(self.party_address.id, None)
#    self.deleteNewPerson()
#  def testCreateInvalidPersonValidPartyAddressUpdateDelete(self):
#    self.insertNewPerson(valid=False)
#    self.insertRelatedPartyAddress(valid=False)
#    self.assertEqual(self.party_address.id, None)
#    self.deleteNewPerson()
#  def tearDown(self):
#    from elixir import session
#    self.block(lambda:self.address.delete())
#    self.block(lambda:self.city.delete())
#    self.block(lambda:self.country.delete())
#    self.block(lambda:session.flush([self.country, self.city, self.address]))

class EditorsTest(ModelThreadTestCase):
    """
  Test the basic functionality of the editors :

  - get_value
  - set_value
  - support for ValueLoading
  """

    images_path = static_images_path

    from camelot.view.controls import editors
    from camelot.view.proxy import ValueLoading

    def test_ChartEditor(self):
        import math
        from camelot.container import chartcontainer
        editor = self.editors.ChartEditor()
        x_data = [x/100.0 for x in range(1, 700, 1)]
        y_data = [math.sin(x) for x in x_data]
        plot = chartcontainer.PlotContainer( x_data, y_data )
        editor.set_value( plot )
        editor.setMaximumSize( 400, 200 )
        self.grab_widget( editor, 'editable' )
        editor.set_field_attributes(editable=False)
        self.grab_widget( editor, 'disabled' )
        
    def test_DateEditor(self):
        import datetime
        editor = self.editors.DateEditor()
        self.assertEqual( editor.get_value(), self.ValueLoading )
        editor.set_value( None )
        self.assertEqual( editor.get_value(), None )
        editor.set_value( datetime.date(1980, 12, 31) )
        self.grab_default_states( editor )
        self.assertEqual( editor.get_value(), datetime.date(1980, 12, 31) )
        editor.set_value( self.ValueLoading )
        self.assertEqual( editor.get_value(), self.ValueLoading )

    def test_TextLineEditor(self):
        editor = self.editors.TextLineEditor(parent=None, length=10)
        self.assertEqual( editor.get_value(), self.ValueLoading )
        editor.set_value( u'za coś tam' )
        self.grab_default_states( editor )
        self.assertEqual( editor.get_value(), u'za coś tam' )
        editor.set_value( self.ValueLoading )
        self.assertEqual( editor.get_value(), self.ValueLoading )
        editor = self.editors.TextLineEditor(parent=None, length=10)
        editor.set_field_attributes( editable=False )
        self.assertEqual( editor.get_value(), self.ValueLoading )
        editor.set_value( u'za coś tam' )
        self.assertEqual( editor.get_value(), u'za coś tam' )
        editor.set_value( self.ValueLoading )
        self.assertEqual( editor.get_value(), self.ValueLoading )
        editor.set_value( None )
        self.assertEqual( editor.get_value(), None )
        # pretend the user has entered some text
        editor.setText( u'foo' )
        self.assertTrue( editor.get_value() != None )
        
    def grab_default_states( self, editor ):
        editor.set_field_attributes( editable = True, background_color=ColorScheme.green )
        self.grab_widget( editor, 'editable_background_color')
        
        editor.set_field_attributes( editable = False, tooltip = 'tooltip' )
        self.grab_widget( editor, 'disabled_tooltip')

        editor.set_field_attributes( editable = False, background_color=ColorScheme.green )
        self.grab_widget( editor, 'disabled_background_color' )

        editor.set_field_attributes( editable = True )
        self.grab_widget( editor, 'editable' )

        editor.set_field_attributes( editable = False )
        self.grab_widget( editor, 'disabled' )

        editor.set_field_attributes( editable = True, tooltip = 'tooltip')
        self.grab_widget( editor, 'editable_tooltip')

    def test_LocalFileEditor( self ):
        editor = self.editors.LocalFileEditor( parent=None )
        self.assertEqual( editor.get_value(), self.ValueLoading )
        editor.set_value( '/home/lancelot/quests.txt' )
        self.grab_default_states( editor )
        self.assertEqual( editor.get_value(), '/home/lancelot/quests.txt' )
        editor.set_value( self.ValueLoading )
        self.assertEqual( editor.get_value(), self.ValueLoading )
        
    def test_StarEditor(self):
        editor = self.editors.StarEditor(parent=None, maximum=5)
        self.assertEqual( editor.get_value(), self.ValueLoading )
        editor.set_value( 4 )
        self.grab_default_states( editor )
        self.assertEqual( editor.get_value(), 4 )
        editor.set_value( self.ValueLoading )
        self.assertEqual( editor.get_value(), self.ValueLoading )

    def test_SmileyEditor(self):
        editor = self.editors.SmileyEditor(parent=None)
        self.assertEqual( editor.get_value(), self.ValueLoading )
        editor.set_value( 'face-kiss' )
        self.grab_default_states( editor )
        self.assertEqual( editor.get_value(), 'face-kiss' )
        editor.set_value( self.ValueLoading )
        self.assertEqual( editor.get_value(), self.ValueLoading )
        
    def test_BoolEditor(self):
        editor = self.editors.BoolEditor(parent=None, editable=False, nullable=True)
        self.assertEqual( editor.get_value(), self.ValueLoading )
        editor.set_value( True )
        self.grab_default_states( editor )
        self.assertEqual( editor.get_value(), True )
        editor.set_value( False )
        self.assertEqual( editor.get_value(), False )
        editor.set_value( self.ValueLoading )
        self.assertEqual( editor.get_value(), self.ValueLoading )
        editor = self.editors.BoolEditor(parent=None, editable=False)
        self.assertEqual( editor.get_value(), self.ValueLoading )
        editor.set_value( True )
        self.assertEqual( editor.get_value(), True )
        editor.set_value( False )
        self.assertEqual( editor.get_value(), False )
        editor.set_value( self.ValueLoading )
        self.assertEqual( editor.get_value(), self.ValueLoading )
        editor.set_value( True )
        editor.set_enabled( True )

    def test_CodeEditor(self):
        editor = self.editors.CodeEditor(parent=None, parts=['AAA', '999'])
        self.assertEqual( editor.get_value(), self.ValueLoading )
        editor.set_value( ['XYZ', '123'] )
        self.grab_default_states( editor )
        self.assertEqual( editor.get_value(), ['XYZ', '123'] )
        editor.set_value( self.ValueLoading )
        self.assertEqual( editor.get_value(), self.ValueLoading )
        editor.set_value( self.ValueLoading )
        self.assertEqual( editor.get_value(), self.ValueLoading )

    def test_ColorEditor(self):
        editor = self.editors.ColorEditor(parent=None, editable=True)
        self.assertEqual( editor.get_value(), self.ValueLoading )
        editor.set_value( (255, 200, 255, 255) )
        self.grab_default_states( editor )
        self.assertEqual( editor.get_value(), (255, 200, 255, 255) )
        editor.set_value( self.ValueLoading )
        self.assertEqual( editor.get_value(), self.ValueLoading )

    def test_ColoredFloatEditor(self):
        editor = self.editors.ColoredFloatEditor(parent=None, editable=True)
        self.assertEqual( editor.get_value(), self.ValueLoading )
        editor.set_value( 0.0 )
        self.assertEqual( editor.get_value(), 0.0 )
        editor.set_value( 3.14 )
        self.grab_default_states( editor )
        self.assertEqual( editor.get_value(), 3.14 )
        editor.set_value( self.ValueLoading )
        self.assertEqual( editor.get_value(), self.ValueLoading )

    def test_ChoicesEditor(self):
        editor = self.editors.ChoicesEditor(parent=None, editable=True)
        choices1 = [(1,u'A'), (2,u'B'), (3,u'C')]
        editor.set_choices( choices1 )
        self.assertEqual( editor.get_value(), self.ValueLoading )
        editor.set_value( 2 )
        self.assertEqual(editor.get_choices(), choices1 )
        self.grab_default_states( editor )
        self.assertEqual( editor.get_value(), 2 )
        # now change the choices
        choices2 = [(4,u'D'), (5,u'E'), (6,u'F')]
        editor.set_choices( choices2 )
        self.assertEqual(editor.get_choices(), choices2 + [(2,u'B')])
        editor.set_value( self.ValueLoading )
        self.assertEqual( editor.get_value(), self.ValueLoading )

    def test_FileEditor(self):
        editor = self.editors.FileEditor(parent=None, editable=True)
        self.assertEqual( editor.get_value(), self.ValueLoading )
        editor.set_value( self.ValueLoading )
        self.assertEqual( editor.get_value(), self.ValueLoading )
        self.grab_default_states( editor )

    def test_DateTimeEditor(self):
        import datetime
        editor = self.editors.DateTimeEditor(parent=None, editable=True)
        self.assertEqual( editor.get_value(), self.ValueLoading )
        editor.set_value( datetime.datetime(2009, 7, 19, 21, 5, 10, 0) )
        self.assertEqual( editor.get_value(), datetime.datetime(2009, 7, 19, 21, 5, 0 ) )
        self.grab_default_states( editor )
        editor.set_value( self.ValueLoading )
        self.assertEqual( editor.get_value(), self.ValueLoading )

    def test_FloatEditor(self):
        editor = self.editors.FloatEditor(parent=None, 
                                          prefix='prefix')
        self.assertEqual( editor.get_value(), self.ValueLoading )
        editor.set_value( 0.0 )
        self.assertEqual( editor.get_value(), 0.0 )
        editor.set_value( 3.14 )
        self.grab_default_states( editor )
        self.assertEqual( editor.get_value(), 3.14 )
        editor.set_value( self.ValueLoading )
        self.assertEqual( editor.get_value(), self.ValueLoading )
        editor = self.editors.FloatEditor(parent=None,  
                                          suffix='suffix')
        self.assertEqual( editor.get_value(), self.ValueLoading )
        editor.set_value( 0.0 )
        self.assertEqual( editor.get_value(), 0.0 )
        editor.set_value( 3.14 )
        self.assertEqual( editor.get_value(), 3.14 )
        editor.set_value( self.ValueLoading )
        self.assertEqual( editor.get_value(), self.ValueLoading )
        editor.set_value( 5.45 )
        editor.set_value( None )
        self.assertEqual( editor.get_value(), None )
        # pretend the user has entered something
        editor.spinBox.setValue( 0.0 )
        self.assertTrue( editor.get_value() != None )
        # verify if the calculator button is turned off
        editor = self.editors.FloatEditor(parent=None, 
                                          calculator=False)
        editor.set_field_attributes( editable=True )
        editor.set_value( 3.14 )
        self.grab_widget( editor, 'no_calculator' )
        self.assertTrue( editor.calculatorButton.isHidden() )
        
    def test_ImageEditor(self):
        editor = self.editors.ImageEditor(parent=None, editable=True)
        self.assertEqual( editor.get_value(), self.ValueLoading )
        editor.set_value( self.ValueLoading )
        self.assertEqual( editor.get_value(), self.ValueLoading )
        self.grab_default_states( editor )

    def test_IntegerEditor(self):
        editor = self.editors.IntegerEditor(parent=None, editable=True)
        self.assertEqual( editor.get_value(), self.ValueLoading )
        editor.set_value( 0 )
        self.assertEqual( editor.get_value(), 0 )
        editor.set_value( 3 )
        self.grab_default_states( editor )
        self.assertEqual( editor.get_value(), 3 )
        editor.set_value( self.ValueLoading )
        self.assertEqual( editor.get_value(), self.ValueLoading )
        editor.set_value( None )
        # pretend the user changed the value
        editor.spinBox.setValue( 0 )
        self.assertEqual( editor.get_value(), 0 )
        editor.set_value( None )
        self.assertEqual( editor.get_value(), None )
        # turn off the calculator
        editor = self.editors.IntegerEditor(parent=None, 
                                            calculator=False)
        editor.set_field_attributes( editable=True )
        editor.set_value( 3.14 )
        self.grab_widget( editor, 'no_calculator' )
        self.assertTrue( editor.calculatorButton.isHidden() )
        
    def test_NoteEditor(self):
        editor = self.editors.NoteEditor(parent=None)
        editor.set_value('A person with this name already exists')
        self.grab_widget( editor )
        self.grab_default_states( editor )

    def test_LabelEditor(self):
        editor = self.editors.LabelEditor(parent=None)
        editor.set_value('Dynamic label')
        self.grab_default_states( editor )

    def test_LanguageEditor(self):
        editor = self.editors.LanguageEditor(parent=None)
        self.assertEqual( editor.get_value(), self.ValueLoading )
        editor.set_value( 'en_US' )
        self.grab_default_states( editor )
        self.assertEqual( editor.get_value(), 'en_US' )
        editor.set_value( self.ValueLoading )
        self.assertEqual( editor.get_value(), self.ValueLoading )

    def test_Many2OneEditor(self):
        editor = self.editors.Many2OneEditor(parent=None)
        self.grab_default_states( editor )
        
    def test_RichTextEditor(self):
        editor = self.editors.RichTextEditor(parent=None)
        self.assertEqual( editor.get_value(), self.ValueLoading )
        editor.set_value( u'<h1>Rich Text Editor</h1>' )
        self.grab_default_states( editor )
        self.assertTrue( u'Rich Text Editor' in editor.get_value() )
        editor.set_value( self.ValueLoading )
        self.assertEqual( editor.get_value(), self.ValueLoading )

    def test_TimeEditor(self):
        import datetime
        editor = self.editors.TimeEditor(parent=None, editable=True)
        self.assertEqual( editor.get_value(), self.ValueLoading )
        editor.set_value( datetime.time(21, 5, 0) )
        self.grab_default_states( editor )
        self.assertEqual( editor.get_value(), datetime.time(21, 5, 0) )
        editor.set_value( self.ValueLoading )
        self.assertEqual( editor.get_value(), self.ValueLoading )

    def test_TextEditEditor(self):
        editor = self.editors.TextEditEditor(parent=None, editable=True)
        self.assertEqual( editor.get_value(), self.ValueLoading )
        editor.set_value( 'Plain text' )
        self.grab_default_states( editor )
        self.assertEqual( editor.get_value(), 'Plain text' )
        editor.set_value( self.ValueLoading )
        self.assertEqual( editor.get_value(), self.ValueLoading )
        
    def test_VirtualAddressEditor(self):
        editor = self.editors.VirtualAddressEditor(parent=None)
        self.assertEqual( editor.get_value(), self.ValueLoading )
        editor.set_value( ('im','test') )
        self.grab_default_states( editor )
        self.assertEqual( editor.get_value(),  ('im','test') )
        editor.set_value( self.ValueLoading )
        self.assertEqual( editor.get_value(), self.ValueLoading )

    def test_MonthsEditor(self):
        editor = self.editors.MonthsEditor(parent=None)
        self.assertEqual(editor.get_value(), self.ValueLoading)
        editor.set_value(12)
        self.grab_default_states( editor )
        self.assertEqual(editor.get_value(),  12)
        editor.set_value(self.ValueLoading)
        self.assertEqual(editor.get_value(), self.ValueLoading)

from camelot.view import forms

class FormTest(ModelThreadTestCase):

    images_path = static_images_path

    def setUp(self):
        ModelThreadTestCase.setUp(self)
        from camelot.view.controls.formview import FormEditors
        from elixir import entities
        self.entities = [e for e in entities]
        from camelot.admin.application_admin import ApplicationAdmin
        from camelot.view.proxy.queryproxy import QueryTableProxy
        from camelot.model.authentication import Person
        from camelot_example.model import Movie
        self.app_admin = ApplicationAdmin()
        self.movie_admin = self.app_admin.get_related_admin( Movie )
        
        self.movie_model = QueryTableProxy( self.movie_admin, 
                                            lambda:Movie.query,
                                            self.movie_admin.get_fields )
        
        widget_mapper = QtGui.QDataWidgetMapper()
        widget_mapper.setModel( self.movie_model )
        widget_mapper.setItemDelegate( self.movie_model.getItemDelegate() )
        self.widgets = FormEditors( self.movie_admin.get_fields(),
                                    widget_mapper,
                                    self.movie_model.getItemDelegate(),
                                    self.movie_admin )
        
        self.person_entity = Person
        self.collection_getter = lambda:[Person()]

    def tearDown(self):
        #
        # The global list of entities should remain clean for subsequent tests
        #
        from elixir import entities
        for e in entities:
            if e not in self.entities:
                entities.remove(e)

    def test_form(self):
        from snippet.form.simple_form import Movie
        self.grab_widget(Movie.Admin.form_display.render(self.widgets))

    def test_tab_form(self):
        form = forms.TabForm([('First tab', ['title', 'short_description']),
                              ('Second tab', ['director', 'releasedate'])])
        self.grab_widget(form.render(self.widgets))

    def test_group_box_form(self):
        form = forms.GroupBoxForm('Movie', ['title', 'short_description'])
        self.grab_widget(form.render(self.widgets))

    def test_grid_form(self):
        form = forms.GridForm([['title', 'short_description'], ['director','releasedate']])
        self.grab_widget(form.render(self.widgets))

    def test_vbox_form(self):
        form = forms.VBoxForm([['title', 'short_description'], ['director', 'releasedate']])
        self.grab_widget(form.render(self.widgets))

    def test_hbox_form(self):
        form = forms.HBoxForm([['title', 'short_description'], ['director', 'releasedate']])
        self.grab_widget(form.render(self.widgets))

    def test_nested_form(self):
        from snippet.form.nested_form import Admin
        person_admin = Admin(self.app_admin, self.person_entity)
        self.grab_widget( person_admin.create_new_view() )

    def test_inherited_form(self):
        from snippet.form.inherited_form import InheritedAdmin
        person_admin = InheritedAdmin(self.app_admin, self.person_entity)
        self.grab_widget( person_admin.create_new_view() )

    def test_custom_layout(self):
        from snippet.form.custom_layout import Admin
        person_admin = Admin(self.app_admin, self.person_entity)
        self.grab_widget( person_admin.create_new_view() )

class DelegateTest(ModelThreadTestCase):
    """Test the basic functionallity of the delegates :
  - createEditor
  - setEditorData
  - setModelData
  """

    from camelot.view.controls import delegates
    from camelot.view.controls import editors

    def setUp(self):
        super(DelegateTest, self).setUp()
        self.kwargs = dict(editable=True)
        self.option = QtGui.QStyleOptionViewItem()
        # set version to 5 to indicate the widget will appear on a
        # a form view and not on a table view, so it should not
        # set its background
        self.option.version = 5

    def grab_delegate(self, delegate, data, suffix='editable'):
        import sys
        from camelot.view.controls.tableview import TableWidget
        from PyQt4.QtCore import Qt

        model = QStandardItemModel(1, 1)
        index = model.index(0, 0, QModelIndex())
        model.setData( index, QtCore.QVariant( data ) )
        model.setData( index, QtCore.QVariant( QtGui.QColor('white') ), Qt.BackgroundRole )
        model.setData( index, QtCore.QVariant( dict(editable=True) ), Qt.UserRole )

        option = QtGui.QStyleOptionViewItem()

        if suffix == 'editable':
            self.assertTrue(delegate.sizeHint(option, index).height()>1)

        tableview = TableWidget()
        tableview.setModel(model)
        tableview.setItemDelegate(delegate)

        test_case_name = sys._getframe(1).f_code.co_name[4:]

        for state_name, state in zip(('selected', 'unselected'),
                                     (QStyle.State_Selected, QStyle.State_None)):
            tableview.adjustSize()

            if state == QStyle.State_Selected:
                tableview.selectionModel().select(index, QItemSelectionModel.Select)
            else:
                tableview.selectionModel().select(index, QItemSelectionModel.Clear)

            cell_size = tableview.visualRect(index).size()

            headers_size = QSize(tableview.verticalHeader().width(),
                                 tableview.horizontalHeader().height())

            tableview.setHorizontalScrollBarPolicy( Qt.ScrollBarAlwaysOff )
            tableview.setVerticalScrollBarPolicy( Qt.ScrollBarAlwaysOff )

            tableview.resize(cell_size + headers_size)

            # TODO checks if path exists
            delegate_images_path = os.path.join(static_images_path, 'delegates')
            if not os.path.exists(delegate_images_path):
                os.makedirs(delegate_images_path)
            pixmap = QPixmap.grabWidget(tableview)
            pixmap.save(os.path.join(delegate_images_path, '%s_%s_%s.png'%(test_case_name, state_name, suffix)),
                        'PNG')

    def testPlainTextDelegate(self):
        delegate = self.delegates.PlainTextDelegate(parent=None,
                                                    length=30,
                                                    editable=True)
        editor = delegate.createEditor(None, self.option, None)
        self.assertEqual(editor.maxLength(), 30)
        self.assertTrue(isinstance(editor, self.editors.TextLineEditor))
        self.grab_delegate(delegate, 'Plain Text')
        delegate = self.delegates.PlainTextDelegate(parent=None,
                                                    length=20,
                                                    editable=False)
        editor = delegate.createEditor(None, self.option, None)
        self.assertTrue(isinstance(editor, self.editors.TextLineEditor))
        self.grab_delegate(delegate, 'Plain Text', 'disabled')

    def testTextEditDelegate(self):
        from PyQt4.QtGui import QTextEdit
        delegate = self.delegates.TextEditDelegate(parent=None, **self.kwargs)
        editor = delegate.createEditor(None, self.option, None)
        self.assertTrue(isinstance(editor, QTextEdit))
        self.grab_delegate(delegate, 'Plain Text')
        delegate = self.delegates.TextEditDelegate(parent=None, editable=False)
        editor = delegate.createEditor(None, self.option, None)
        self.assertTrue(isinstance(editor, QTextEdit))
        self.grab_delegate(delegate, 'Plain Text', 'disabled')

    def testRichTextDelegate(self):
        delegate = self.delegates.RichTextDelegate(parent=None, **self.kwargs)
        editor = delegate.createEditor(None, self.option, None)
        self.assertTrue(isinstance(editor, self.editors.RichTextEditor))
        self.grab_delegate(delegate, '<b>Rich Text</b>')
        delegate = self.delegates.RichTextDelegate(parent=None, editable=False)
        editor = delegate.createEditor(None, self.option, None)
        self.assertTrue(isinstance(editor, self.editors.RichTextEditor))
        self.grab_delegate(delegate, '<b>Rich Text</b>', 'disabled')

    def testBoolDelegate(self):
        delegate = self.delegates.BoolDelegate(parent=None, **self.kwargs)
        editor = delegate.createEditor(None, self.option, None)
        self.assertTrue(isinstance(editor, self.editors.BoolEditor))
        self.grab_delegate(delegate, True)
        delegate = self.delegates.BoolDelegate(parent=None, editable=False)
        editor = delegate.createEditor(None, self.option, None)
        self.assertTrue(isinstance(editor, self.editors.BoolEditor))
        self.grab_delegate(delegate, True, 'disabled')

    def testDateDelegate(self):
        from datetime import date
        delegate = self.delegates.DateDelegate(parent=None, **self.kwargs)
        editor = delegate.createEditor(None, self.option, None)
        self.assertTrue(isinstance(editor, self.editors.DateEditor))
        date = date.today()
        self.grab_delegate(delegate, date)
        delegate = self.delegates.DateDelegate(parent=None, editable=False)
        editor = delegate.createEditor(None, self.option, None)
        self.assertTrue(isinstance(editor, self.editors.DateEditor))
        date = date.today()
        self.grab_delegate(delegate, date, 'disabled')
        #TODO  !!

    def testDateTimeDelegate(self):
        from datetime import datetime
        delegate = self.delegates.DateTimeDelegate(parent=None, **self.kwargs)
        editor = delegate.createEditor(None, self.option, None)
        self.assertTrue(isinstance(editor, self.editors.DateTimeEditor))
        DateTime = datetime.now()
        self.grab_delegate(delegate, DateTime)
        delegate = self.delegates.DateTimeDelegate(parent=None, editable=False)
        editor = delegate.createEditor(None, self.option, None)
        self.assertTrue(isinstance(editor, self.editors.DateTimeEditor))
        self.grab_delegate(delegate, DateTime, 'disabled')

    def testEnumerationDelegate(self):
        choices = [('a','A'), ('b','B')]
        delegate = self.delegates.EnumerationDelegate(parent=None, choices=choices)
        self.grab_delegate(delegate, 'a')
        delegate = self.delegates.EnumerationDelegate(parent=None, choices=choices, editable=False)
        self.grab_delegate(delegate, 'a', 'disabled')

    def testLocalFileDelegate(self):
        delegate = self.delegates.LocalFileDelegate(parent=None)
        self.grab_delegate(delegate, '/home/lancelot/quests.txt')
        delegate = self.delegates.LocalFileDelegate(parent=None, editable=False)
        self.grab_delegate(delegate, '/home/lancelot/quests.txt', 'disabled')
        
    def testLabelDelegate(self):
        delegate = self.delegates.LabelDelegate(parent=None)
        self.grab_delegate(delegate, 'dynamic label')
        delegate = self.delegates.LabelDelegate(parent=None, editable=False)
        self.grab_delegate(delegate, 'dynamic label', 'disabled')

    def testNoteDelegate(self):
        delegate = self.delegates.NoteDelegate(parent=None)
        self.grab_delegate(delegate, 'important note')
        delegate = self.delegates.NoteDelegate(parent=None, editable=False)
        self.grab_delegate(delegate, 'important note', 'disabled')
        
    def testMonthsDelegate(self):
        delegate = self.delegates.MonthsDelegate(parent=None)
        self.grab_delegate(delegate, 15)
        delegate = self.delegates.MonthsDelegate(parent=None, editable=False)
        self.grab_delegate(delegate, 15, 'disabled')

    def testMany2OneDelegate(self):
        delegate = self.delegates.Many2OneDelegate(parent=None, admin=object())
        self.grab_delegate(delegate, None)
        delegate = self.delegates.Many2OneDelegate(parent=None, editable=False, admin=object())
        self.grab_delegate(delegate, None, 'disabled')
        
    def testOne2ManyDelegate(self):
        delegate = self.delegates.One2ManyDelegate(parent=None, admin=object())
        self.grab_delegate(delegate, [])
        delegate = self.delegates.One2ManyDelegate(parent=None, editable=False, admin=object())
        self.grab_delegate(delegate, [], 'disabled')
        
    def testTimeDelegate(self):
        from datetime import time
        delegate = self.delegates.TimeDelegate(parent=None, editable=True)
        editor = delegate.createEditor(None, self.option, None)
        self.assertTrue(isinstance(editor, self.editors.TimeEditor))
        time = time(10, 30, 15)
        self.grab_delegate(delegate, time)
        delegate = self.delegates.TimeDelegate(parent=None, editable=False)
        editor = delegate.createEditor(None, self.option, None)
        self.assertTrue(isinstance(editor, self.editors.TimeEditor))
        #time = time(10, 30, 15)
        self.grab_delegate(delegate, time, 'disabled')

    def testIntegerDelegate(self):
        delegate = self.delegates.IntegerDelegate(parent=None, editable=True)
        editor = delegate.createEditor(None, self.option, None)
        self.assertTrue(isinstance(editor, self.editors.IntegerEditor))
        self.grab_delegate(delegate, 3)
        delegate = self.delegates.IntegerDelegate(parent=None, editable=False)
        editor = delegate.createEditor(None, self.option, None)
        self.assertTrue(isinstance(editor, self.editors.IntegerEditor))
        self.grab_delegate(delegate, 0, 'disabled')

    def testIntervalsDelegate(self):
        from camelot.container import IntervalsContainer, Interval
        intervals = IntervalsContainer(0, 24, [Interval(8, 18, 'work', QtGui.QColor(255,0,0,255)), Interval(19, 21, 'play', QtGui.QColor(0,255,0,255))])
        delegate = self.delegates.IntervalsDelegate(parent=None, editable=True)
        self.grab_delegate(delegate, intervals)
        delegate = self.delegates.IntervalsDelegate(parent=None, editable=False)
        self.grab_delegate(delegate, intervals, 'disabled')

    def testFloatDelegate(self):
        from camelot.core.constants import camelot_minfloat, camelot_maxfloat
        delegate = self.delegates.FloatDelegate(parent=None, suffix='euro', editable=True)
        editor = delegate.createEditor(None, self.option, None)
        self.assertTrue(isinstance(editor, self.editors.FloatEditor))
        self.assertEqual(editor.spinBox.minimum(), camelot_minfloat)
        self.assertEqual(editor.spinBox.maximum(), camelot_maxfloat)
        self.grab_delegate(delegate, 3.145)
        delegate = self.delegates.FloatDelegate(parent=None, prefix='prefix', editable=False)
        editor = delegate.createEditor(None, self.option, None)
        self.assertTrue(isinstance(editor, self.editors.FloatEditor))
        self.assertEqual(editor.spinBox.minimum(), camelot_minfloat)
        self.assertEqual(editor.spinBox.maximum(), camelot_maxfloat)
        self.grab_delegate(delegate, 0, 'disabled')

    def testColoredFloatDelegate(self):
        delegate = self.delegates.ColoredFloatDelegate(parent=None, precision=3, editable=True)
        editor = delegate.createEditor(None, self.option, None)
        self.assertTrue(isinstance(editor, self.editors.ColoredFloatEditor))
        self.grab_delegate(delegate, 3.14456)
        delegate = self.delegates.ColoredFloatDelegate(parent=None, editable=False)
        editor = delegate.createEditor(None, self.option, None)
        self.assertTrue(isinstance(editor, self.editors.ColoredFloatEditor))
        self.grab_delegate(delegate, 3.1, 'disabled')

    def testStarDelegate(self):
        delegate = self.delegates.StarDelegate(parent=None, **self.kwargs)
        editor = delegate.createEditor(None, self.option, None)
        self.assertTrue(delegate.maximum, 5)
        self.assertTrue(isinstance(editor, self.editors.StarEditor))
        self.grab_delegate(delegate, 5)
        delegate = self.delegates.StarDelegate(parent=None, editable=False)
        editor = delegate.createEditor(None, self.option, None)
        self.assertTrue(delegate.maximum, 5)
        self.assertTrue(isinstance(editor, self.editors.StarEditor))
        self.grab_delegate(delegate, 5, 'disabled')

    def testSmileyDelegate(self):
        delegate = self.delegates.SmileyDelegate(parent=None, **self.kwargs)
        editor = delegate.createEditor(None, self.option, None)
        self.assertTrue(isinstance(editor, self.editors.SmileyEditor))
        self.grab_delegate(delegate, 'face-glasses')
        delegate = self.delegates.SmileyDelegate(parent=None, editable=False)
        editor = delegate.createEditor(None, self.option, None)
        self.assertTrue(isinstance(editor, self.editors.SmileyEditor))
        self.grab_delegate(delegate, 'face-glasses', 'disabled')

    def testFileDelegate(self):
        from camelot.core.files.storage import StoredFile
        delegate = self.delegates.FileDelegate(parent=None)
        file = StoredFile(None, 'agreement.pdf')
        editor = delegate.createEditor(None, self.option, None)
        self.assertTrue(isinstance(editor, self.editors.FileEditor))
        self.grab_delegate(delegate, file)
        delegate = self.delegates.FileDelegate(parent=None, editable=False)
        self.grab_delegate(delegate, file, 'disabled')

    def testColorDelegate(self):
        delegate = self.delegates.ColorDelegate(parent=None, **self.kwargs)
        editor = delegate.createEditor(None, self.option, None)
        self.assertTrue(isinstance(editor, self.editors.ColorEditor))
        color = [255, 255, 0]
        self.grab_delegate(delegate, color)
        delegate = self.delegates.ColorDelegate(parent=None, editable=False)
        editor = delegate.createEditor(None, self.option, None)
        self.assertTrue(isinstance(editor, self.editors.ColorEditor))
        self.grab_delegate(delegate, color, 'disabled')

    def testCodeDelegate(self):
        delegate = self.delegates.CodeDelegate(parent=None, parts=['99','AA'], **self.kwargs)
        editor = delegate.createEditor(None, self.option, None)
        self.assertTrue(isinstance(editor, self.editors.CodeEditor))
        self.grab_delegate(delegate, ['76','AB'])
        delegate = self.delegates.CodeDelegate(parent=None, parts=['99','AA', '99', 'AA'], editable=False)
        editor = delegate.createEditor(None, self.option, None)
        self.assertTrue(isinstance(editor, self.editors.CodeEditor))
        self.grab_delegate(delegate, ['76','AB', '12', '34'], 'disabled')

    def testCurrencyDelegate(self):
        delegate = self.delegates.CurrencyDelegate(parent=None, suffix='euro', **self.kwargs)
        editor = delegate.createEditor(None, self.option, None)
        self.assertTrue(isinstance(editor, self.editors.FloatEditor))
        self.grab_delegate(delegate, 1000000.12)
        delegate = self.delegates.CurrencyDelegate(parent=None, prefix='prefix', editable=False)
        self.grab_delegate(delegate, 1000000.12, 'disabled')

    def testComboBoxDelegate(self):
        CHOICES = (('1','A'), ('2','B'), ('3','C'))
        delegate = self.delegates.ComboBoxDelegate(parent=None,
                                                   choices=CHOICES,
                                                   **self.kwargs)
        editor = delegate.createEditor(None, self.option, None)
        self.assertTrue(isinstance(editor, self.editors.ChoicesEditor))
        self.grab_delegate(delegate, 1)
        delegate = self.delegates.ComboBoxDelegate(parent=None,
                                                   choices=CHOICES,
                                                   editable=False)
        self.grab_delegate(delegate, 1, 'disabled')

    def testImageDelegate(self):
        delegate = self.delegates.ImageDelegate(parent=None, **self.kwargs)
        editor = delegate.createEditor(None, self.option, None)
        self.assertTrue(isinstance(editor, self.editors.ImageEditor))

    def testVirtualAddressDelegate(self):
        delegate = self.delegates.VirtualAddressDelegate(parent=None,
                                                         **self.kwargs)
        editor = delegate.createEditor(None, self.option, None)
        self.assertTrue(isinstance(editor, self.editors.VirtualAddressEditor))
        self.grab_delegate(delegate, ('email', 'project-camelot@conceptive.be'))
        delegate = self.delegates.VirtualAddressDelegate(parent=None, editable=False)
        self.grab_delegate(delegate, ('email', 'project-camelot@conceptive.be'), 'disabled')

    def testMonthsDelegate(self):
        delegate = self.delegates.MonthsDelegate(parent=None, **self.kwargs)
        editor = delegate.createEditor(None, self.option, None)
        self.assertTrue(isinstance(editor, self.editors.MonthsEditor))
        self.grab_delegate(delegate, 12)
        delegate = self.delegates.MonthsDelegate(parent=None, editable=False)
        self.grab_delegate(delegate, 12, 'disabled')


class FilterTest(ModelThreadTestCase):
    """Test the filters in the table view"""

    images_path = static_images_path
    from camelot.view import filters

    group_box_filter = filters.GroupBoxFilter('state')
    combo_box_filter = filters.ComboBoxFilter('state')
    name_and_options = ('Organization', [('Nokia',None), ('Apple',None)])

    def test_group_box_filter(self):
        self.grab_widget(self.group_box_filter.render(None, *self.name_and_options))

    def test_combo_box_filter(self):
        self.grab_widget(self.combo_box_filter.render(None, *self.name_and_options))

    def test_filter_list(self):
        from camelot.view.controls.filterlist import FilterList
        items = [(self.group_box_filter, self.name_and_options),
                 (self.combo_box_filter, self.name_and_options)]
        filter_list = FilterList(items, parent=None)
        self.grab_widget(filter_list)

    def test_filter_list_in_table_view(self):
        from camelot.view.controls.tableview import TableView
        from camelot.model.authentication import Person
        from camelot.admin.application_admin import ApplicationAdmin
        app_admin = ApplicationAdmin()
        person_admin = Person.Admin(app_admin, Person)
        table_view = TableView(person_admin)
        items = [(self.group_box_filter, self.name_and_options),
                 (self.combo_box_filter, self.name_and_options)]
        table_view.set_filters_and_actions((items, None))

class ControlsTest(ModelThreadTestCase):
    """Test some basic controls"""

    images_path = static_images_path

    def setUp(self):
        super(ControlsTest, self).setUp()
        from camelot_example.application_admin import MyApplicationAdmin
        self.app_admin = MyApplicationAdmin()

    def wait_for_animation( self ):
        # wait a while to make sure all animations are finished
        for i in range(10):
            time.sleep(0.1)
            self.app.processEvents()
        
    def test_table_view(self):
        from camelot.view.controls.tableview import TableView
        from camelot.model.authentication import Person
        widget = TableView(self.app_admin.get_entity_admin(Person))
        self.grab_widget(widget)

    def test_navigation_pane(self):
        from camelot.view.controls import navpane2
        self.wait_for_animation()
        widget = navpane2.NavigationPane( self.app_admin,
                                          workspace = None,
                                          parent = None )
        widget.set_sections( self.app_admin.get_sections() )
        self.grab_widget(widget)

    def test_main_window(self):
        from camelot.view.mainwindow import MainWindow
        widget = MainWindow(self.app_admin)
        self.wait_for_animation()
        self.grab_widget(widget)

    def test_status_bar(self):
        from camelot.view.controls.statusbar import StatusBar
        status_bar = StatusBar(None)
        status_bar.busy_widget.set_busy(True)
        self.grab_widget(status_bar)

    def test_search_control(self):
        from camelot.view.controls.search import SimpleSearchControl
        search = SimpleSearchControl(None)
        self.grab_widget(search)

    def test_header_widget(self):
        from camelot.model.authentication import City
        from camelot.view.controls.tableview import HeaderWidget
        person_admin = self.app_admin.get_entity_admin(City)
        header = HeaderWidget(parent=None, admin=person_admin)
        header.expand_search_options()
        self.grab_widget(header)
        
    def test_desktop_workspace(self):
        from camelot.view.workspace import DesktopWorkspace
        from camelot.admin.action import Action
        from camelot.view.art import Icon
       
        action1 = Action()
        action1.icon=Icon('tango/32x32/places/network-server.png')
        action2 = Action()
        action2.icon=Icon('tango/32x32/places/user-trash.png')
        action3 = Action()
        action3.icon=Icon('tango/32x32/places/start-here.png')
        desktopWorkspace = DesktopWorkspace(self.app_admin, None)
        actions = [ action1, action2, action3 ]

        desktopWorkspace._background_widget.set_actions( actions )
        self.grab_widget(desktopWorkspace)

    def test_progress_dialog( self ):
        from camelot.view.controls.progress_dialog import ProgressDialog
        dialog = ProgressDialog( 'Import cover images' )
        self.grab_widget( dialog )
        
    def test_user_exception(self):
        from camelot.view.controls.exception import register_exception, ExceptionDialog
        try:
            #begin user_exception
            from camelot.core.exception import UserException

            raise UserException( text = "Could not burn movie to non empty DVD",
                                 resolution = "Insert an empty DVD and retry" )
            #end user_exception
        except Exception, e:
            pass

        exc_info = register_exception(logger, 'unit test', e)
        dialog = ExceptionDialog( exc_info )
        self.grab_widget( dialog )   

class CamelotEntityViewsTest(EntityViewsTest):
    """Test the views of all the Entity subclasses"""

    images_path = static_images_path

class SnippetsTest(ModelThreadTestCase):

    images_path = static_images_path

    def test_simple_plot(self):
        from snippet.chart.simple_plot import Wave
        from camelot.view.proxy.collection_proxy import CollectionProxy
        wave = Wave()
        admin = Wave.Admin(None, Wave)
        proxy = CollectionProxy(admin, lambda:[wave], admin.get_fields )
        form = admin.create_form_view('Wave', proxy, 0, None)
        form.setMaximumSize( 400, 200 )
        self.grab_widget(form)

    def test_advanced_plot(self):
        from snippet.chart.advanced_plot import Wave
        from camelot.view.proxy.collection_proxy import CollectionProxy
        wave = Wave()
        #wave.phase = '2.89'
        admin = Wave.Admin(object(), Wave)
        proxy = CollectionProxy(admin, lambda:[wave], admin.get_fields )
        form = admin.create_form_view('Wave', proxy, 0, None)
        form.setMaximumSize( 400, 200 )
        self.grab_widget(form)
        
    def test_fields_with_actions(self):
        from snippet.fields_with_actions import Coordinate
        from camelot.view.proxy.collection_proxy import CollectionProxy
        coordinate = Coordinate()
        admin = Coordinate.Admin(None, Coordinate)
        proxy = CollectionProxy(admin, lambda:[coordinate], admin.get_fields )
        form = admin.create_form_view('Coordinate', proxy, 0, None)
        self.grab_widget(form)

    def test_fields_with_tooltips(self):
        from snippet.fields_with_tooltips import Coordinate
        from camelot.view.proxy.collection_proxy import CollectionProxy
        coordinate = Coordinate()
        admin = Coordinate.Admin(None, Coordinate)
        proxy = CollectionProxy(admin, lambda:[coordinate], admin.get_fields )
        form = admin.create_form_view('Coordinate', proxy, 0, None)
        self.grab_widget(form)

    def test_entity_validator(self):
        from camelot.view.proxy.collection_proxy import CollectionProxy
        from camelot.model.authentication import Person
        from camelot.admin.application_admin import ApplicationAdmin
        from snippet.entity_validator import PersonValidator, Admin
        app_admin = ApplicationAdmin()
        person_admin = Admin(app_admin, Person)
        proxy = CollectionProxy(person_admin, lambda:[Person()], person_admin.get_columns)
        validator = PersonValidator(person_admin, proxy)
        self.mt.post(lambda:validator.isValid(0))
        self.process()
        self.assertEqual(len(validator.validityMessages(0)), 3)
        self.grab_widget(validator.validityDialog(0, parent=None))

    def test_background_color(self):
        from camelot.view.proxy.collection_proxy import CollectionProxy
        from camelot.model.authentication import Person
        from camelot.admin.application_admin import ApplicationAdmin
        from snippet.background_color import Admin
        app_admin = ApplicationAdmin()
        person_admin = Admin(app_admin, Person)
        proxy = CollectionProxy(person_admin, lambda:[Person(first_name='John', last_name='Cleese'),
                                                      Person(first_name='eric', last_name='Idle')],
                                                       person_admin.get_columns)
        from camelot.view.controls.editors.one2manyeditor import One2ManyEditor
        editor = One2ManyEditor(admin=person_admin)
        editor.set_value(proxy)
        self.process()
        self.grab_widget(editor)

class CamelotSchemaTest(SchemaTest):

    images_path = static_images_path




#  ============================================================================
#
#  Copyright (C) 2007-2011 Conceptive Engineering bvba. All rights reserved.
#  www.conceptive.be / project-camelot@conceptive.be
#
#  This file is part of the Camelot Library.
#
#  This file may be used under the terms of the GNU General Public
#  License version 2.0 as published by the Free Software Foundation
#  and appearing in the file license.txt included in the packaging of
#  this file.  Please review this information to ensure GNU
#  General Public Licensing requirements will be met.
#
#  If you are unsure which license is appropriate for your use, please
#  visit www.python-camelot.com or contact project-camelot@conceptive.be
#
#  This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
#  WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#
#  For use of this library in commercial applications, please contact
#  project-camelot@conceptive.be
#
#  ============================================================================
'''
Created on Jan 7, 2010

@author: tw55413
'''

from PyQt4 import QtCore, QtGui

from camelot.core.utils import ugettext_lazy as _

class ProgressPage(QtGui.QWizardPage):
    """Generic progress page for a wizard.
    
    Subclass and reimplement the run method.  And within this run method,
    regulary emit the update_progress_signal and the update_maximum_signal.
    
    the update_maximum_signal should have as its single argument an integer
    value indicating the maximum of the progress bar.
    
    the update_progress_signal should two arguments, the first is an integer
    indicating the current position of the progress bar, and the second is
    a string telling the user what is going on.
    
    If required, set the title and sub_title class attribute to change the
    text displayed to the user.
    """
    
    update_progress_signal = QtCore.pyqtSignal(int, str)
    update_maximum_signal = QtCore.pyqtSignal(int)
    
    title = _('Action in progress')
    sub_title = _('Please wait for completion')
    
    def __init__(self, parent=None):
        super(ProgressPage, self).__init__( parent )
        self.update_progress_signal.connect( self.update_progress )
        self.update_maximum_signal.connect( self.update_maximum )
        self._complete = False
        self.setTitle(unicode(self.title))
        self.setSubTitle(unicode(self.sub_title))
        layout = QtGui.QVBoxLayout()
        progress = QtGui.QProgressBar(self)
        progress.setObjectName('progress')
        progress.setMinimum(0)
        progress.setMaximum(1)
        label = QtGui.QTextEdit(self)
        label.setObjectName('label')
        label.setSizePolicy( QtGui.QSizePolicy.Expanding,
                             QtGui.QSizePolicy.Expanding )
        label.setReadOnly(True)
        layout.addWidget(progress)
        layout.addWidget(label)
        self.setLayout(layout)
    
    def isComplete(self):
        return self._complete
    
    @QtCore.pyqtSlot(int)
    def update_maximum(self, maximum):
        progress = self.findChild(QtGui.QWidget, 'progress' )
        if progress:
            progress.setMaximum(maximum)
    
    @QtCore.pyqtSlot(int, str)
    def update_progress(self, value, label):
        progress_widget = self.findChild(QtGui.QWidget, 'progress' )
        if progress_widget:
            progress_widget.setValue(value)
        label_widget = self.findChild(QtGui.QWidget, 'label' )
        if label_widget:            
            label_widget.setHtml(unicode(label))

    def exception(self, args):
        self.finished()
        from camelot.view.controls.exception import model_thread_exception_message_box
        model_thread_exception_message_box(args)
        
    def finished(self):
        self._complete = True
        progress_widget = self.findChild(QtGui.QWidget, 'progress' )
        if progress_widget:
            progress_widget.setMaximum(1)
            progress_widget.setValue(1)
        self.completeChanged.emit()     
        
    def run(self):
        """
        This method contains the actual action, that will be run in the model thread.
        
        Reimplement this method, while regulary emiting update_progress_signal and
        update_maximum_signal to keep the progress bar moving.
        """
        pass
                      
    def initializePage(self):
        from camelot.view.model_thread import post
        post(self.run, self.finished, self.exception)



#  ============================================================================
#
#  Copyright (C) 2007-2011 Conceptive Engineering bvba. All rights reserved.
#  www.conceptive.be / project-camelot@conceptive.be
#
#  This file is part of the Camelot Library.
#
#  This file may be used under the terms of the GNU General Public
#  License version 2.0 as published by the Free Software Foundation
#  and appearing in the file license.txt included in the packaging of
#  this file.  Please review this information to ensure GNU
#  General Public Licensing requirements will be met.
#
#  If you are unsure which license is appropriate for your use, please
#  visit www.python-camelot.com or contact project-camelot@conceptive.be
#
#  This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
#  WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#
#  For use of this library in commercial applications, please contact
#  project-camelot@conceptive.be
#
#  ============================================================================
'''
Created on Sep 9, 2009

@author: tw55413
'''
import logging
import sys
logger = logging.getLogger('camelot.view.model_thread.signal_slot_model_thread')

from PyQt4 import QtCore

from camelot.view.model_thread import AbstractModelThread, gui_function, setup_model
from camelot.core.threading import synchronized
from camelot.view.controls.exception import register_exception

class Task(QtCore.QObject):

    finished = QtCore.pyqtSignal(object)
    exception = QtCore.pyqtSignal(object)

    def __init__(self, request, name='', args=()):
        QtCore.QObject.__init__(self)
        self._request = request
        self._name = name
        self._args = args

    def clear(self):
        """clear this tasks references to other objects"""
        self._request = None
        self._name = None
        self._args = None

    def execute(self):
        logger.debug('executing %s' % (self._name))
        try:
            result = self._request( *self._args )
            self.finished.emit( result )
        #
        # don't handle StopIteration as a normal exception, but return a new
        # instance of StopIteration (in order to not keep alive a stack trace),
        # and to signal to the caller that an iterator has ended
        #
        except StopIteration:
            self.finished.emit( StopIteration() )
        except Exception, e:
            exc_info = register_exception(logger, 'exception caught in model thread while executing %s'%self._name, e)
            self.exception.emit( exc_info )
            # the stack might contain references to QT objects which could be kept alive this way
            sys.exc_clear()
        except:
            logger.error( 'unhandled exception in model thread' )
            sys.exc_clear()

class TaskHandler(QtCore.QObject):
    """A task handler is an object that handles tasks that appear in a queue,
    when its handle_task method is called, it will sequentially handle all tasks
    that are in the queue.
    """

    task_handler_busy_signal = QtCore.pyqtSignal(bool)

    def __init__(self, queue):
        """:param queue: the queue from which to pop a task when handle_task
        is called"""
        QtCore.QObject.__init__(self)
        self._mutex = QtCore.QMutex()
        self._queue = queue
        self._tasks_done = []
        self._busy = False
        logger.debug("TaskHandler created.")

    def busy(self):
        """:return True/False: indicating if this task handler is busy"""
        return self._busy

    @QtCore.pyqtSlot()
    def handle_task(self):
        """Handle all tasks that are in the queue"""
        self._busy = True
        self.task_handler_busy_signal.emit( True )
        task = self._queue.pop()
        while task:
            task.execute()
            # we keep track of the tasks done to prevent them being garbage collected
            # apparently when they are garbage collected, they are recycled, but their
            # signal slot connections seem to survive this recycling.
            # @todo: this should be investigated in more detail, since we are causing
            #        a deliberate memory leak here
            #
            # not keeping track of the tasks might result in corruption
            #
            # see : http://www.riverbankcomputing.com/pipermail/pyqt/2011-August/030452.html
            #
            task.clear()
            self._tasks_done.append(task)
            task = self._queue.pop()
        self.task_handler_busy_signal.emit( False )
        self._busy = False

class SignalSlotModelThread( AbstractModelThread ):
    """A model thread implementation that uses signals and slots
    to communicate between the model thread and the gui thread

    there is no explicit model thread verification on these methods,
    since this model thread might not be THE model thread.
    """

    task_available = QtCore.pyqtSignal()

    def __init__( self, setup_thread = setup_model ):
        """
        @param setup_thread: function to be called at startup of the thread to initialize
        everything, by default this will setup the model.  set to None if nothing should
        be done.
        """
        from camelot.view.model_thread.garbage_collector import GarbageCollector
        super(SignalSlotModelThread, self).__init__( setup_thread )
        self._task_handler = None
        self._mutex = QtCore.QMutex()
        self._request_queue = []
        self._connected = False
        self._setup_busy = True
        GarbageCollector( self )

    def run( self ):
        self.logger.debug( 'model thread started' )
        self._task_handler = TaskHandler(self)
        self._task_handler.task_handler_busy_signal.connect(self._thread_busy, QtCore.Qt.QueuedConnection)
        self._thread_busy(True)
        try:
            self._setup_thread()
        except Exception, e:
            exc_info = register_exception(logger, 'Exception when setting up the SignalSlotModelThread', e)
            self.setup_exception_signal.emit( exc_info )
        self._thread_busy(False)
        self.logger.debug('thread setup finished')
        # Some tasks might have been posted before the signals were connected to the task handler,
        # so once force the handling of tasks
        self._task_handler.handle_task()
        self._setup_busy = False
        self.exec_()
        self.logger.debug('model thread stopped')

    @QtCore.pyqtSlot( bool )
    def _thread_busy(self, busy_state):
        self.thread_busy_signal.emit( busy_state )

    @synchronized
    def post( self, request, response = None, exception = None, args = () ):
        if not self._connected and self._task_handler:
            # creating this connection in the model thread throws QT exceptions
            self.task_available.connect( self._task_handler.handle_task, QtCore.Qt.QueuedConnection )
            self._connected = True
        # response should be a slot method of a QObject
        if response:
            name = '%s -> %s.%s'%(request.__name__, response.im_self.__class__.__name__, response.__name__)
        else:
            name = request.__name__
        task = Task(request, name=name, args=args)
        # QObject::connect is a thread safe function
        if response:
            assert response.im_self != None
            assert isinstance(response.im_self, QtCore.QObject)
            # verify if the response has been defined as a slot
            #assert hasattr(response, '__pyqtSignature__')
            task.finished.connect( response, QtCore.Qt.QueuedConnection )
        if exception:
            task.exception.connect( exception, QtCore.Qt.QueuedConnection )
        # task.moveToThread(self)
        # only put the task in the queue when it is completely set up
        self._request_queue.append(task)
        #print 'task created --->', id(task)
        self.task_available.emit()

    @synchronized
    def stop( self ):
        self.quit()
        return True
    
    @synchronized
    def pop( self ):
        """Pop a task from the queue, return None if the queue is empty"""
        if len(self._request_queue):
            task = self._request_queue.pop(0)
            return task

    @synchronized
    def busy( self ):
        """Return True or False indicating wether either the model or the
        gui thread is doing something"""
        while not self._task_handler:
            import time
            time.sleep(1)
        app = QtCore.QCoreApplication.instance()
        return app.hasPendingEvents() or len(self._request_queue) or self._task_handler.busy() or self._setup_busy

    @gui_function
    def wait_on_work(self):
        """Wait for all work to be finished, this function should only be used
        to do unit testing and such, since it will block the calling thread until
        all work is done"""
        app = QtCore.QCoreApplication.instance()
        while self.busy():
            app.processEvents()

#    app = QCoreApplication.instance()
#    waiting = True
#    while waiting:
#      waiting = False
#      if app.hasPendingEvents():
#        app.processEvents()
#        waiting = True





#  ============================================================================
#
#  Copyright (C) 2007-2011 Conceptive Engineering bvba. All rights reserved.
#  www.conceptive.be / project-camelot@conceptive.be
#
#  This file is part of the Camelot Library.
#
#  This file may be used under the terms of the GNU General Public
#  License version 2.0 as published by the Free Software Foundation
#  and appearing in the file license.txt included in the packaging of
#  this file.  Please review this information to ensure GNU
#  General Public Licensing requirements will be met.
#
#  If you are unsure which license is appropriate for your use, please
#  visit www.python-camelot.com or contact project-camelot@conceptive.be
#
#  This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
#  WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#
#  For use of this library in commercial applications, please contact
#  project-camelot@conceptive.be
#
#  ============================================================================
import datetime

from PyQt4 import QtCore
from PyQt4.QtCore import Qt

from customdelegate import CustomDelegate, DocumentationMetaclass
from camelot.view.controls import editors
from camelot.core.utils import create_constant_function, variant_to_pyobject
from camelot.view.proxy import ValueLoading

class TimeDelegate(CustomDelegate):
 
    __metaclass__ = DocumentationMetaclass
   
    editor = editors.TimeEditor
      
    def __init__(self, parent=None, editable=True, **kwargs):
        CustomDelegate.__init__(self, parent, editable)
        locale = QtCore.QLocale()
        self.time_format = locale.timeFormat(locale.ShortFormat)
        
    def paint(self, painter, option, index):
        painter.save()
        self.drawBackground(painter, option, index)
        value = variant_to_pyobject( index.model().data( index, Qt.EditRole ) )
        
        value_str = u''
        if value not in (None, ValueLoading):
            time = QtCore.QTime(value.hour, value.minute, value.second)
            value_str = time.toString(self.time_format)

        self.paint_text(painter, option, index, value_str)
        painter.restore()
      
    def setModelData(self, editor, model, index):
        value = editor.time()
        t = datetime.time(hour=value.hour(),
                          minute=value.minute(),
                          second=value.second())
        model.setData(index, create_constant_function(t))





#  ============================================================================
#
#  Copyright (C) 2007-2011 Conceptive Engineering bvba. All rights reserved.
#  www.conceptive.be / project-camelot@conceptive.be
#
#  This file is part of the Camelot Library.
#
#  This file may be used under the terms of the GNU General Public
#  License version 2.0 as published by the Free Software Foundation
#  and appearing in the file license.txt included in the packaging of
#  this file.  Please review this information to ensure GNU
#  General Public Licensing requirements will be met.
#
#  If you are unsure which license is appropriate for your use, please
#  visit www.python-camelot.com or contact project-camelot@conceptive.be
#
#  This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
#  WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#
#  For use of this library in commercial applications, please contact
#  project-camelot@conceptive.be
#
#  ============================================================================

"""Encapsulates various ms office schemes"""

Office2003BlueStyle = {
  'SplitterDark' : '(21, 28, 171)',
  'SplitterLight' : '(129, 168, 226)',
  'BorderColor' : '(21, 28, 171)',
  'TextColor' : '(0, 0, 0)',
  'ButtonNormalBegin' : '(201, 223, 251)',
  'ButtonNormalMiddleBegin' : '(165, 196, 239)',
  'ButtonNormalMiddleEnd' : '(165, 196, 239)',
  'ButtonNormalEnd' : '(129, 168, 226)',
  'ButtonMouseOverBegin' : '(255, 253, 216)',
  'ButtonMouseOverMiddleBegin' : '(251, 222, 152)',
  'ButtonMouseOverMiddleEnd' : '(251, 222, 152)',
  'ButtonMouseOverEnd' : '(248, 194, 95)',
  'SelectedTextColor' : '(0, 0, 0)',
  'ButtonSelectedBegin' : '(251, 228, 144)',
  'ButtonSelectedMiddleBegin' : '(245, 190, 85)',
  'ButtonSelectedMiddleEnd' : '(245, 190, 85)',
  'ButtonSelectedEnd' : '(240, 153, 25)',
  'CaptionBegin' : '(201, 223, 251)',
  'CaptionEnd' : '(129, 168, 226)',
  'CaptionDescBegin' : '(201, 223, 251)',
  'CaptionDescEnd' : '(129, 168, 226)',
  'CaptionTextColor' : '(21, 28, 171)',
  'ContextMenuBackColor' : '(253, 250, 255)'
}

Office2003SilverStyle = {
  'SplitterDark' : '(150, 148, 178)',
  'SplitterLight' : '(223, 224, 234)',
  'BorderColor' : '(100, 97, 135)',
  'TextColor' : '(0, 0, 0)',
  'ButtonNormalBegin' : '(223, 224, 234)',
  'ButtonNormalMiddleBegin' : '(190, 190, 209)',
  'ButtonNormalMiddleEnd' : '(190, 190, 209)',
  'ButtonNormalEnd' : '(150, 148, 178)',
  'ButtonMouseOverBegin' : '(255, 253, 216)',
  'ButtonMouseOverMiddleBegin' : '(251, 222, 152)',
  'ButtonMouseOverMiddleEnd' : '(251, 222, 152)',
  'ButtonMouseOverEnd' : '(248, 194, 95)',
  'SelectedTextColor' : '(0, 0, 0)',
  'ButtonSelectedBegin' : '(251, 228, 144)',
  'ButtonSelectedMiddleBegin' : '(245, 190, 85)',
  'ButtonSelectedMiddleEnd' : '(245, 190, 85)',
  'ButtonSelectedEnd' : '(240, 153, 25)',
  'CaptionBegin' : '(223, 224, 234)',
  'CaptionEnd' : '(100, 97, 135)',
  'CaptionDescBegin' : '(223, 224, 234)',
  'CaptionDescEnd' : '(150, 148, 178)',
  'CaptionTextColor' : '(0, 0, 0)',
  'ContextMenuBackColor' : '(253, 250, 255)'
}

Office2003OliveStyle = {
  'SplitterDark' : '(182, 196, 144)',
  'SplitterLight' : '(233, 239, 205)',
  'BorderColor' : '(134, 161, 105)',
  'TextColor' : '(0, 0, 0)',
  'ButtonNormalBegin' : '(233, 239, 205)',
  'ButtonNormalMiddleBegin' : '(208, 218, 176)',
  'ButtonNormalMiddleEnd' : '(208, 218, 176)',
  'ButtonNormalEnd' : '(182, 196, 144)',
  'ButtonMouseOverBegin' : '(255, 253, 216)',
  'ButtonMouseOverMiddleBegin' : '(251, 222, 152)',
  'ButtonMouseOverMiddleEnd' : '(251, 222, 152)',
  'ButtonMouseOverEnd' : '(248, 194, 95)',
  'SelectedTextColor' : '(0, 0, 0)',
  'ButtonSelectedBegin' : '(251, 228, 144)',
  'ButtonSelectedMiddleBegin' : '(245, 190, 85)',
  'ButtonSelectedMiddleEnd' : '(245, 190, 85)',
  'ButtonSelectedEnd' : '(240, 153, 25)',
  'CaptionBegin' : '(233, 239, 205)',
  'CaptionEnd' : '(134, 161, 105)',
  'CaptionDescBegin' : '(233, 239, 205)',
  'CaptionDescEnd' : '(182, 196, 144)',
  'CaptionTextColor' : '(0, 0, 0)',
  'ContextMenuBackColor' : '(253, 250, 255)'
}

Office2007BlueStyle = {
  'SplitterDark' : '(192, 219, 255)',
  'SplitterLight' : '(227, 239, 255)',
  'BorderColor' : '(101, 147, 207)',
  'TextColor' : '(32, 77, 137)',
  'ButtonNormalBegin' : '(227, 239, 255)',
  'ButtonNormalMiddleBegin' : '(196, 221, 255)',
  'ButtonNormalMiddleEnd' : '(173, 209, 255)',
  'ButtonNormalEnd' : '(192, 219, 255)',
  'ButtonMouseOverBegin' : '(255, 254, 228)',
  'ButtonMouseOverMiddleBegin' : '(255, 232, 167)',
  'ButtonMouseOverMiddleEnd' : '(255, 215, 103)',
  'ButtonMouseOverEnd' : '(255, 230, 158)',
  'SelectedTextColor' : '(0, 0, 0)',
  'ButtonSelectedBegin' : '(255, 217, 170)',
  'ButtonSelectedMiddleBegin' : '(255, 187, 110)',
  'ButtonSelectedMiddleEnd' : '(255, 171, 63)',
  'ButtonSelectedEnd' : '(254, 225, 122)',
  'ButtonSelectedOverBegin' : '(255, 189, 105)',
  'ButtonSelectedOverMiddleBegin' : '(255, 172, 66)',
  'ButtonSelectedOverMiddleEnd' : '(251, 140, 60)',
  'ButtonSelectedOverEnd' : '(254, 211, 100)',
  'CaptionBegin' : '(227, 239, 255)',
  'CaptionEnd' : '(173, 209, 255)',
  'CaptionDescBegin' : '(227, 239, 255)',
  'CaptionDescEnd' : '(192, 219, 255)',
  'CaptionTextColor' : '(21, 66, 139)',
  'ContextMenuBackColor' : '(227, 239, 255)'
}

Office2007BlackStyle = {
  'SplitterDark' : '(199, 203, 209)',
  'SplitterLight' : '(219, 222, 226)',
  'BorderColor' : '(76, 83, 92)',
  'TextColor' : '(76, 83, 92)',
  'ButtonNormalBegin' : '(248, 248, 249)',
  'ButtonNormalMiddleBegin' : '(223, 226, 228)',
  'ButtonNormalMiddleEnd' : '(199, 203, 209)',
  'ButtonNormalEnd' : '(219, 222, 226)',
  'ButtonMouseOverBegin' : '(255, 254, 228)',
  'ButtonMouseOverMiddleBegin' : '(255, 232, 167)',
  'ButtonMouseOverMiddleEnd' : '(255, 215, 103)',
  'ButtonMouseOverEnd' : '(255, 230, 158)',
  'SelectedTextColor' : '(0, 0, 0)',
  'ButtonSelectedBegin' : '(255, 217, 170)',
  'ButtonSelectedMiddleBegin' : '(255, 187, 110)',
  'ButtonSelectedMiddleEnd' : '(255, 171, 63)',
  'ButtonSelectedEnd' : '(254, 225, 122)',
  'CaptionBegin' : '(240, 241, 242)',
  'CaptionEnd' : '(189, 193, 200)',
  'CaptionDescBegin' : '(240, 241, 242)',
  'CaptionDescEnd' : '(189, 193, 200)',
  'CaptionTextColor' : '(0, 0, 0)',
  'ContextMenuBackColor' : '(248, 248, 249)'
}

Office2007SilverStyle = {
  'SplitterDark' : '(119, 118, 151)',
  'SplitterLight' : '(168, 167, 191)',
  'BorderColor' : '(111, 112, 116)',
  'TextColor' : '(111, 112, 116)',
  'ButtonNormalBegin' : '(235, 238, 250)',
  'ButtonNormalMiddleBegin' : '(214, 218, 228)',
  'ButtonNormalMiddleEnd' : '(197, 199, 204)',
  'ButtonNormalEnd' : '(212, 216, 226)',
  'ButtonMouseOverBegin' : '(255, 254, 228)',
  'ButtonMouseOverMiddleBegin' : '(255, 232, 167)',
  'ButtonMouseOverMiddleEnd' : '(255, 215, 103)',
  'ButtonMouseOverEnd' : '(255, 230, 158)',
  'SelectedTextColor' : '(0, 0, 0)',
  'ButtonSelectedBegin' : '(255, 217, 170)',
  'ButtonSelectedMiddleBegin' : '(255, 187, 110)',
  'ButtonSelectedMiddleEnd' : '(255, 171, 63)',
  'ButtonSelectedEnd' : '(254, 225, 122)',
  'CaptionBegin' : '(246, 247, 248)',
  'CaptionEnd' : '(218, 223, 230)',
  'CaptionDescBegin' : '(246, 247, 248)',
  'CaptionDescEnd' : '(218, 223, 230)',
  'CaptionTextColor' : '(0, 0, 0)',
  'ContextMenuBackColor' : '(235, 238, 250)'
}

class Scheme(object):
    def __init__(self, styledict=Office2007BlueStyle):
        self.styledict = styledict

    def color(self, r, g, b):
        return 'rgb(%s, %s, %s)' % (r, g, b)

    def bordercolor(self):
        return 'rgb%(BorderColor)s' % self.styledict

    def textcolor(self):
        return 'rgb%(TextColor)s' % self.styledict

    def selectedcolor(self):
        return 'rgb%(SelectedTextColor)s' % self.styledict

    def captiontextcolor(self):
        return 'rgb%(CaptionTextColor)s' % self.styledict

    def twocolorgradient(self, rgb1, rgb2):
        s = 'qlineargradient(spread:pad, ' \
            'x1:0.5, y1:0, x2:0.5, y2:1, ' \
            'stop:0 rgba%s, 255), ' \
            'stop:1 rgba%s, 255))' % (rgb1, rgb2)

        return s

    def fourcolorgradient(self, rgb1, rgb2, rgb3, rgb4):
        s = 'qlineargradient(spread:pad, ' \
            'x1:0.5, y1:0, x2:0.5, y2:1, ' \
            'stop:0 rgba%s, 255), ' \
            'stop:0.4 rgba%s, 255), ' \
            'stop:0.401 rgba%s, 255), ' \
            'stop:1 rgba%s, 255))' % (rgb1, rgb2, rgb3, rgb4)

        return s

    def normalbackground(self):
        rgb1 = self.styledict['ButtonNormalBegin'].rstrip(')')
        rgb2 = self.styledict['ButtonNormalMiddleBegin'].rstrip(')')
        rgb3 = self.styledict['ButtonNormalMiddleEnd'].rstrip(')')
        rgb4 = self.styledict['ButtonNormalEnd'].rstrip(')')

        return self.fourcolorgradient(rgb1, rgb2, rgb3, rgb4)

    def hoveredbackground(self):
        rgb1 = self.styledict['ButtonMouseOverBegin'].rstrip(')')
        rgb2 = self.styledict['ButtonMouseOverMiddleBegin'].rstrip(')')
        rgb3 = self.styledict['ButtonMouseOverMiddleEnd'].rstrip(')')
        rgb4 = self.styledict['ButtonMouseOverEnd'].rstrip(')')

        return self.fourcolorgradient(rgb1, rgb2, rgb3, rgb4)

    def selectedbackground(self, inverted=False):
        if inverted:
            rgb1 = self.styledict['ButtonSelectedOverBegin'].rstrip(')')
            rgb2 = self.styledict['ButtonSelectedOverMiddleBegin'].rstrip(')')
            rgb3 = self.styledict['ButtonSelectedOverMiddleEnd'].rstrip(')')
            rgb4 = self.styledict['ButtonSelectedOverEnd'].rstrip(')')
        else:
            rgb1 = self.styledict['ButtonSelectedBegin'].rstrip(')')
            rgb2 = self.styledict['ButtonSelectedMiddleBegin'].rstrip(')')
            rgb3 = self.styledict['ButtonSelectedMiddleEnd'].rstrip(')')
            rgb4 = self.styledict['ButtonSelectedEnd'].rstrip(')')

        return self.fourcolorgradient(rgb1, rgb2, rgb3, rgb4)

    def captionbackground(self):
        rgb1 = self.styledict['CaptionBegin'].rstrip(')')
        rgb2 = self.styledict['CaptionEnd'].rstrip(')')

        return self.twocolorgradient(rgb1, rgb2)

scheme = Scheme()
defaultUI = Office2007BlueStyle





#  ============================================================================
#
#  Copyright (C) 2007-2011 Conceptive Engineering bvba. All rights reserved.
#  www.conceptive.be / project-camelot@conceptive.be
#
#  This file is part of the Camelot Library.
#
#  This file may be used under the terms of the GNU General Public
#  License version 2.0 as published by the Free Software Foundation
#  and appearing in the file license.txt included in the packaging of
#  this file.  Please review this information to ensure GNU
#  General Public Licensing requirements will be met.
#
#  If you are unsure which license is appropriate for your use, please
#  visit www.python-camelot.com or contact project-camelot@conceptive.be
#
#  This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
#  WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#
#  For use of this library in commercial applications, please contact
#  project-camelot@conceptive.be
#
#  ============================================================================

from PyQt4 import QtGui

from camelot.admin.action.base import Action, GuiContext, Mode, ModelContext
from camelot.core.utils import ugettext, ugettext_lazy as _
from camelot.view.art import Icon

"""ModelContex, GuiContext and Actions that run in the context of an 
application.
"""

class ApplicationActionModelContext( ModelContext ):
    """The Model context for an :class:`camelot.admin.action.Action`.  On top 
    of the attributes of the :class:`camelot.admin.action.base.ModelContext`, 
    this context contains :
        
    .. attribute:: admin
    
        the application admin.
    """
    
    def __init__( self ):
        super( ApplicationActionModelContext, self ).__init__()
        self.admin = None
        
class ApplicationActionGuiContext( GuiContext ):
    """The GUI context for an :class:`camelot.admin.action.Action`.  On top of 
    the attributes of the :class:`camelot.admin.action.base.GuiContext`, this 
    context contains :
    
    .. attribute:: workspace
    
        the the :class:`camelot.view.workspace.DesktopWorkspace` of the 
        application in which views can be opened or adapted.
        
    .. attribute:: admin
    
        the application admin.
    """
    
    model_context = ApplicationActionModelContext
    
    def __init__( self ):
        super( ApplicationActionGuiContext, self ).__init__()
        self.workspace = None
        self.admin = None
        
    def create_model_context( self ):
        context = super( ApplicationActionGuiContext, self ).create_model_context()
        context.admin = self.admin
        return context
        
    def copy( self ):
        new_context = super( ApplicationActionGuiContext, self ).copy()
        new_context.workspace = self.workspace
        new_context.admin = self.admin
        return new_context
        
class EntityAction( Action ):
    """Generic ApplicationAction that acts upon an Entity class"""

    def __init__( self, 
                  entity_admin ):
        """
        :param entity_admin: an instance of 
            :class:`camelot.admin.entity_admin.EntityAdmin` to be used to
            visualize the entities
        """
        from camelot.admin.entity_admin import EntityAdmin
        assert isinstance( entity_admin, (EntityAdmin,) )
        self._entity_admin = entity_admin
        
class OpenTableView( EntityAction ):
    """An application action that opens a TableView of an Entity

    :param entity_admin: an instance of 
        :class:`camelot.admin.entity_admin.EntityAdmin` to be used to
        visualize the entities
    
    """

    modes = [ Mode( 'new_tab', _('Open in New Tab') ) ]
        
    def get_state( self, model_context ):
        state = super( OpenTableView, self ).get_state( model_context )
        state.verbose_name = self.verbose_name or self._entity_admin.get_verbose_name_plural()
        return state
        
    def gui_run( self, gui_context ):
        table_view = self._entity_admin.create_table_view()
        if gui_context.mode_name == 'new_tab':
            gui_context.workspace.add_view( table_view )
        else:
            gui_context.workspace.set_view( table_view )
        
class OpenNewView( EntityAction ):
    """An application action that opens a new view of an Entity
    
    :param entity_admin: an instance of 
        :class:`camelot.admin.entity_admin.EntityAdmin` to be used to
        visualize the entities
    
    """

    def get_state( self, model_context ):
        state = super( OpenNewView, self ).get_state( model_context )
        state.verbose_name = self.verbose_name or ugettext('New %s')%(self._entity_admin.get_verbose_name())
        state.tooltip = ugettext('Create a new %s')%(self._entity_admin.get_verbose_name())
        return state
        
    def gui_run( self, gui_context ):
        """:return: a new view"""
        from camelot.view.workspace import show_top_level
        form = self._entity_admin.create_new_view(parent=None)
        show_top_level( form, gui_context.workspace )
        
class ShowHelp( Action ):
    """Display the help window"""
    
    shortcut = QtGui.QKeySequence.HelpContents
    icon = Icon('tango/16x16/apps/help-browser.png')
    tooltip = _('Help')
        
def structure_to_application_action(structure, application_admin):
    """Convert a python structure to an ApplicationAction

    :param application_admin: the 
        :class:`camelot.admin.application_admin.ApplicationAdmin` to use to
        create other Admin classes.
    """
    if isinstance(structure, (Action,)):
        return structure
    admin = application_admin.get_related_admin( structure )
    return OpenTableView( admin )



#  ============================================================================
#
#  Copyright (C) 2007-2011 Conceptive Engineering bvba. All rights reserved.
#  www.conceptive.be / project-camelot@conceptive.be
#
#  This file is part of the Camelot Library.
#
#  This file may be used under the terms of the GNU General Public
#  License version 2.0 as published by the Free Software Foundation
#  and appearing in the file license.txt included in the packaging of
#  this file.  Please review this information to ensure GNU
#  General Public Licensing requirements will be met.
#
#  If you are unsure which license is appropriate for your use, please
#  visit www.python-camelot.com or contact project-camelot@conceptive.be
#
#  This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
#  WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#
#  For use of this library in commercial applications, please contact
#  project-camelot@conceptive.be
#
#  ============================================================================
from PyQt4.QtCore import Qt

from camelot.view.controls import editors
from customdelegate import CustomDelegate, DocumentationMetaclass
from camelot.core.utils import variant_to_pyobject

import logging
logger = logging.getLogger( 'camelot.view.controls.delegates.one2manydelegate' )

class One2ManyDelegate( CustomDelegate ):
    """Custom delegate for many 2 one relations
  
  .. image:: /_static/onetomany.png  
  """

    __metaclass__ = DocumentationMetaclass

    def __init__( self, parent = None, **kwargs ):
        super( One2ManyDelegate, self ).__init__( parent=parent, **kwargs )
        logger.debug( 'create one2manycolumn delegate' )
        self.kwargs = kwargs

    def createEditor( self, parent, option, index ):
        logger.debug( 'create a one2many editor' )
        editor = editors.One2ManyEditor( parent = parent, **self.kwargs )
        self.setEditorData( editor, index )
        return editor

    def setEditorData( self, editor, index ):
        logger.debug( 'set one2many editor data' )
        model = variant_to_pyobject( index.data( Qt.EditRole ) )
        editor.set_value( model )
        field_attributes = variant_to_pyobject(index.data(Qt.UserRole))
        editor.set_field_attributes(**field_attributes)

    def setModelData( self, editor, model, index ):
        pass
    
    def render_ooxml( self, value ):
        return []





#  ============================================================================
#
#  Copyright (C) 2007-2011 Conceptive Engineering bvba. All rights reserved.
#  www.conceptive.be / project-camelot@conceptive.be
#
#  This file is part of the Camelot Library.
#
#  This file may be used under the terms of the GNU General Public
#  License version 2.0 as published by the Free Software Foundation
#  and appearing in the file license.txt included in the packaging of
#  this file.  Please review this information to ensure GNU
#  General Public Licensing requirements will be met.
#
#  If you are unsure which license is appropriate for your use, please
#  visit www.python-camelot.com or contact project-camelot@conceptive.be
#
#  This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
#  WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#
#  For use of this library in commercial applications, please contact
#  project-camelot@conceptive.be
#
#  ============================================================================

from camelot.admin.action import ActionStep
from camelot.core.exception import CancelRequest

class UpdateProgress( ActionStep ):
    """
Inform the user about the progress the application is making
while executing an action.  This ActionStep is not blocking.  So it can
be used inside transactions and will result in a minimum of delay when
yielded.  Each time an object is yielded, the progress dialog will be
updated.

.. image:: /_static/controls/progress_dialog.png

:param value: the current step
:param maximum: the maximum number of steps that will be executed. set it
    to 0 to display a busy indicator instead of a progres bar
:param text: the text to be displayed inside the progres bar
:param detail: the text to be displayed below the progres bar, this text is
    appended to the text already there
:param clear_details: clear the details text already there before putting 
    the new detail text.
"""
    blocking = False
    
    def __init__( self,
                  value=0, 
                  maximum=0, 
                  text=None, 
                  detail=None, 
                  clear_details=False ):
        super(UpdateProgress, self).__init__()
        self._value = value
        self._maximum = maximum
        self._text = text
        self._detail = detail
        self._clear_details = clear_details
        
    def gui_run( self, gui_context ):
        """This method will update the progress dialog, if such dialog exists
        within the GuiContext
        
        :param gui_context: a :class:`camelot.admin.action.GuiContext` instance
        """
        progress_dialog = gui_context.progress_dialog
        if progress_dialog:
            if progress_dialog.wasCanceled():
                progress_dialog.reset()
                raise CancelRequest()
            progress_dialog.setMaximum( self._maximum )
            progress_dialog.setValue( self._value )
            if self._text != None:
                progress_dialog.setLabelText( unicode(self._text) )



#  ============================================================================
#
#  Copyright (C) 2007-2011 Conceptive Engineering bvba. All rights reserved.
#  www.conceptive.be / project-camelot@conceptive.be
#
#  This file is part of the Camelot Library.
#
#  This file may be used under the terms of the GNU General Public
#  License version 2.0 as published by the Free Software Foundation
#  and appearing in the file license.txt included in the packaging of
#  this file.  Please review this information to ensure GNU
#  General Public Licensing requirements will be met.
#
#  If you are unsure which license is appropriate for your use, please
#  visit www.python-camelot.com or contact project-camelot@conceptive.be
#
#  This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
#  WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#
#  For use of this library in commercial applications, please contact
#  project-camelot@conceptive.be
#
#  ============================================================================

from PyQt4 import QtGui, QtCore

from camelot.admin.action import ActionStep
from camelot.core.exception import CancelRequest
from camelot.core.utils import ugettext_lazy as _
from camelot.core.utils import ugettext
from camelot.view.art import Icon
from camelot.view.controls.standalone_wizard_page import StandaloneWizardPage
from camelot.view.model_thread import post

class ChangeObjectDialog( StandaloneWizardPage ):
    """A dialog to change an object.  This differs from a FormView in that
    it does not contains Actions, and has an OK button that is enabled when
    the object is valid.
    
    :param obj: The object to change
    :param admin: The admin class used to create a form
    
    .. image:: /_static/actionsteps/change_object.png
    """
        
    def __init__( self, 
                  obj, 
                  admin,
                  title =  _('Please complete'),
                  subtitle = _('Complete the form and press the OK button'),
                  icon = Icon('tango/22x22/categories/preferences-system.png'),
                  parent=None, 
                  flags=QtCore.Qt.WindowFlags(0) ):
        from camelot.view.controls.formview import FormWidget
        from camelot.view.proxy.collection_proxy import CollectionProxy
        super(ChangeObjectDialog, self).__init__( '', parent, flags )
        
        self.setWindowTitle( admin.get_verbose_name() )
        self.set_banner_logo_pixmap( icon.getQPixmap() )
        self.set_banner_title( unicode(title) )
        self.set_banner_subtitle( unicode(subtitle) )
        self.banner_widget().setStyleSheet('background-color: white;')
        
        model = CollectionProxy(admin, lambda:[obj], admin.get_fields)
        validator = model.get_validator()
        layout = QtGui.QVBoxLayout()
        form_widget = FormWidget( parent=self, admin=admin )
        layout.addWidget( form_widget )
        validator.validity_changed_signal.connect( self._validity_changed )
        form_widget.set_model( model )
        form_widget.setObjectName( 'form' )
        self.main_widget().setLayout(layout)
    
        cancel_button = QtGui.QPushButton( ugettext('Cancel') )
        ok_button = QtGui.QPushButton( ugettext('OK') )
        ok_button.setObjectName( 'ok' )
        ok_button.setEnabled( False )
        layout = QtGui.QHBoxLayout()
        layout.setDirection( QtGui.QBoxLayout.RightToLeft )
        layout.addWidget( ok_button )
        layout.addWidget( cancel_button )
        layout.addStretch()
        self.buttons_widget().setLayout( layout )
        cancel_button.pressed.connect( self.reject )
        ok_button.pressed.connect( self.accept )
        
        # do inital validation, so the validity changed signal is valid
        self._validity_changed( 0 )
        
    @QtCore.pyqtSlot(int)
    def _validity_changed(self, row):
        form = self.findChild( QtGui.QWidget, 'form' )
        if not form:
            return
        model = form.get_model()
 
        def is_valid():
            return model.get_validator().isValid(0)
        
        post(is_valid, self._change_complete)
        
    def _change_complete(self, complete):
        ok_button = self.findChild( QtGui.QPushButton, 'ok' )
        if ok_button:
            ok_button.setEnabled( complete )

class ChangeObjectsDialog( StandaloneWizardPage ):
    """A dialog to change a list of objects.  This differs from a ListView in 
    that it does not contains Actions, and has an OK button that is enabled when
    all objects are valid.
    
    :param objects: The object to change
    :param admin: The admin class used to create a form
    
    .. image:: /_static/actionsteps/change_object.png
    """

    def __init__( self, 
                  objects, 
                  admin, 
                  parent = None, 
                  flags=QtCore.Qt.WindowFlags(0) ):
        from camelot.view.controls import editors
        from camelot.view.proxy.collection_proxy import CollectionProxy
        
        super(ChangeObjectsDialog, self).__init__( '', parent, flags )
        
        self.set_banner_title( _('Data Preview') )
        self.set_banner_subtitle( _('Please review the data below.') )
        self.banner_widget().setStyleSheet('background-color: white;')
        self.set_banner_logo_pixmap( Icon('tango/32x32/mimetypes/x-office-spreadsheet.png').getQPixmap() )
        model = CollectionProxy( admin, lambda:objects, admin.get_columns)
        self.validator = model.get_validator()
        self.validator.validity_changed_signal.connect( self.update_complete )
        model.layoutChanged.connect( self.validate_all_rows )

        table_widget = editors.One2ManyEditor(
            admin = admin,
            parent = self,
            create_inline = True,
            vertical_header_clickable = False,
        )
        table_widget.set_value( model )
        table_widget.setObjectName( 'table_widget' )
        note = editors.NoteEditor( parent=self )
        note.set_value(None)
        note.setObjectName( 'note' )
        layout = QtGui.QVBoxLayout()
        layout.addWidget( table_widget )
        layout.addWidget( note )
        self.main_widget().setLayout( layout )
    
        cancel_button = QtGui.QPushButton( ugettext('Cancel') )
        ok_button = QtGui.QPushButton( ugettext('OK') )
        ok_button.setObjectName( 'ok' )
        ok_button.setEnabled( False )
        layout = QtGui.QHBoxLayout()
        layout.setDirection( QtGui.QBoxLayout.RightToLeft )
        layout.addWidget( ok_button )
        layout.addWidget( cancel_button )
        layout.addStretch()
        self.buttons_widget().setLayout( layout )
        cancel_button.pressed.connect( self.reject )
        ok_button.pressed.connect( self.accept )
        self.validate_all_rows()

    @QtCore.pyqtSlot()
    def validate_all_rows(self):
        post( self.validator.validate_all_rows, self._all_rows_validated)

    def _all_rows_validated(self, *args):
        self.update_complete( 0 )

    @QtCore.pyqtSlot(int)
    def update_complete(self, row=0):
        complete = (self.validator.number_of_invalid_rows()==0)
        note = self.findChild( QtGui.QWidget, 'note' )
        ok = self.findChild( QtGui.QWidget, 'ok' )
        if note != None and ok != None:
            ok.setEnabled( complete )
            if complete:
                note.set_value( None )
            else:
                note.set_value(_(
                    'Please correct the data above before proceeding with the '
                    'import.<br/>Incorrect cells have a pink background.'
                ))
    
class ChangeObject( ActionStep ):
    """
    Pop up a form for the user to change an object
    
    :param obj: the object to change
    :param admin: an instance of an admin class to use to edit the
        object, None if the default is to be taken
    """
        
    def __init__( self, obj, admin=None ):
        self._obj = obj
        self._admin = admin
        
    def get_object( self ):
        """Use this method to get access to the object to change in unit tests
        
        :return: the object to change
        """
        return self._obj

    def gui_run( self, gui_context ):
        cls = self._obj.__class__
        admin = self._admin or gui_context.admin.get_related_admin( cls )
        dialog = ChangeObjectDialog( self._obj, admin )
        result = dialog.exec_()
        if result == QtGui.QDialog.Rejected:
            raise CancelRequest()
        return self._obj

class ChangeObjects( ActionStep ):
    """
    Pop up a list for the user to change objects
    
    :param objects: a list of objects to change
    :param admin: an instance of an admin class to use to edit the objects.
    """
    
    def __init__( self, objects, admin ):
        self.objects = objects
        self.admin = admin
        
    def get_objects( self ):
        """Use this method to get access to the objects to change in unit tests
        
        :return: the object to change
        """
        return self.objects        
    
    def gui_run( self, gui_context ):
        dialog = ChangeObjectsDialog( self.objects, self.admin )
        result = dialog.exec_()
        if result == QtGui.QDialog.Rejected:
            raise CancelRequest()
        return self.objects
